//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#ifndef __QMOL_ADJUST_TORSION
#define __QMOL_ADJUST_TORSION

#include "array.h"
#include "list.h"
#include "QmolAtom.h"
#include "SelectObject.h"

// Forward definition of molecule
class molecule;

// Keep track of user adjustments to selected torsion angles
class AdjustTorsion{
	private:
		// The two atoms defining the torsion angle
		int tor_a, tor_b;
		
		// A list of atoms affected by the torsion angle
		array<int> affected_atoms;
		
		// The most recent value of the torsion angle
		float last_tor;

		// The molecule to adjust
		molecule *torMol;
	public:
		// Default constructor to make sure that the 
		// vector default constructor is called.
		AdjustTorsion()
		{
			Init();
		};
		
		void Init()
		{
			tor_a = tor_b = -1;
			affected_atoms.Alloc(0);
			last_tor = 0.0;
			torMol = NULL;
		};
		
		inline molecule* GetMol()
		{
			return torMol;
		};

		// Return the most recent value of the torsion angle.
		inline float TorValue()
		{
			return last_tor;
		};
		
		inline int GetAtomA() 
		{
			return tor_a;
		};

		inline int GetAtomB() 
		{
			return tor_b;
		};
		
		inline int IsSelected()
		{
			return (torMol && affected_atoms.Length());
		};
		
		void Set(SelectObj &m_obj);
		
		void Trace(array< array<unsigned int> > &m_molConnect, int a, 
				   array<int> &atom_buffer);

		// Rotate all of the affected atoms
		void Update(float new_tor);
};

#endif // __QMOL_ADJUST_TORSION