//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "Qmol.h"
#include "QmolView.h"

#include "AdjustTorsionDialog.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAdjustTorsionDialog dialog


CAdjustTorsionDialog::CAdjustTorsionDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CAdjustTorsionDialog::IDD, pParent)
{
	m_pView = NULL;
}

CAdjustTorsionDialog::CAdjustTorsionDialog(CView *pView)
{
	m_pView = pView;

	//{{AFX_DATA_INIT(CAdjustTorsionDialog)
	m_tor = _T("");
	//}}AFX_DATA_INIT
}


BOOL CAdjustTorsionDialog::Create()
{
	return CDialog::Create(CAdjustTorsionDialog::IDD);
}

void CAdjustTorsionDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAdjustTorsionDialog)
	DDX_Control(pDX, IDC_TOR_SLIDER, m_slider);
	DDX_Text(pDX, IDC_TOR_LABEL, m_tor);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAdjustTorsionDialog, CDialog)
	//{{AFX_MSG_MAP(CAdjustTorsionDialog)
	ON_WM_HSCROLL()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAdjustTorsionDialog message handlers

// Set the torsion angle
// -PI <= angle <= PI
void CAdjustTorsionDialog::Set(float angle)
{
	if((angle < -PI) || (angle > PI))
		angle = 0.0;

	m_slider.SetPos((int)(180.0*angle/PI + 180.0));

	m_tor.Format("%.3g", PI*(m_slider.GetPos() - 180.0)/180.0);
	UpdateData(FALSE);
}

void CAdjustTorsionDialog::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if(m_pView != NULL){
		m_pView->PostMessage(WM_TOR_ADJUST);
	}

	m_tor.Format("%.3g", PI*(m_slider.GetPos() - 180.0)/180.0);
	UpdateData(FALSE);

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

BOOL CAdjustTorsionDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// Set the range and position of the slider
	// (Note that one can obtain finer control over the slider
	// position by using the arrow keys insead of the mouse!).
	m_slider.SetRange(0, 360); // [-Pi, Pi]
	m_slider.SetTicFreq(20);


	if(m_pView != NULL){
		if(((CQmolView*)m_pView)->tor_adjust.IsSelected()){
			m_slider.SetPos((int)(((CQmolView*)m_pView)->tor_adjust.TorValue()) + 180);
		}
		else{
			// Set the position to the center and disable the slider
			m_slider.SetPos(360);
			m_slider.EnableWindow(FALSE);
		}
	}
	else{
		m_slider.SetPos(360);
	}

	m_tor.Format("%.3g", PI*(m_slider.GetPos() - 180.0)/180.0);
	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAdjustTorsionDialog::OnClose() 
{
	if(m_pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		m_pView->PostMessage(WM_TOR_CLOSE, IDCANCEL);
	}
	else{
		CDialog::OnClose(); // Modal Case
	}
}
