// AtomListDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Qmol.h"
#include "AtomListDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// AtomListDialog dialog


AtomListDialog::AtomListDialog(BOOL m_mask, list<int> &m_selected, 
							   array<BOOL> &m_pool, molecule *m_mol,
							   int &m_showAtom)
	: CDialog(AtomListDialog::IDD, NULL)
{
	ptrSelected = &m_selected;
	ptrShowAtom = &m_showAtom;
	mol = m_mol;

	// Make a list of all available atoms
	int len = m_pool.Length();

	for(int i = 0;i < len;i++){
		// Check the selection state of each atom
		if(m_pool[i] == m_mask){
			pool.Push(i);
		}
	}

	//{{AFX_DATA_INIT(AtomListDialog)
	//}}AFX_DATA_INIT
}


void AtomListDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(AtomListDialog)
	DDX_Control(pDX, IDC_ATOM_LIST, m_list);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(AtomListDialog, CDialog)
	//{{AFX_MSG_MAP(AtomListDialog)
	ON_BN_CLICKED(IDC_ALL_RADIO, OnAllRadio)
	ON_BN_CLICKED(IDC_CA_RADIO, OnCaRadio)
	ON_BN_CLICKED(IDC_HEAVY_RADIO, OnHeavyRadio)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// AtomListDialog message handlers
BOOL AtomListDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	// Select the correct radio button
	CheckRadioButton(IDC_ALL_RADIO,		// First button
					 IDC_CA_RADIO,		// Last bottom	
					 *ptrShowAtom);		// Button to check

	LoadAtomList(*ptrShowAtom);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void AtomListDialog::LoadAtomList(int m_show) 
{
	ListIterator<int> next(pool);
	int *ptrIndex;

	while(!subPool.IsEmpty())
		subPool.Pop();

	// Prepare the subpool
	switch(m_show){
		case IDC_ALL_RADIO:
			while(ptrIndex = next()){
				subPool.Push(*ptrIndex);
			}
			break;
		case IDC_HEAVY_RADIO:
			while(ptrIndex = next()){
				atom &atomRef = mol->atoms[*ptrIndex];

				if(atomRef.elem != ELEM_H)
					subPool.Push(*ptrIndex);
			}
			break;
		case IDC_CA_RADIO:
			while(ptrIndex = next()){
				atom &atomRef = mol->atoms[*ptrIndex];

				if((atomRef.elem == ELEM_C) &&
				   (strcmpi(atomRef.name, "CA") == 0))
					subPool.Push(*ptrIndex);
			}
			break;
	};

	// Remove any existing entries.
	m_list.ResetContent();

	// Load the list
	ListIterator<int> nextSub(subPool);
	int count = 0;
	while(ptrIndex = nextSub()){
		atom &atomRef = mol->atoms[*ptrIndex];
		CString buffer;
		
		buffer.Format("%d:%s:%s%d", 
			atomRef.id,
			atomRef.name,
			atomRef.resName,
			atomRef.resSeq);
		
		m_list.InsertString(count, LPCSTR(buffer));
		count ++;
	}
}

void AtomListDialog::OnOK() 
{
	// Save the list of selected atoms
	
	ListIterator<int> next(subPool);
	int *ptrIndex;
	int count = 0;

	while(ptrIndex = next()){
		if(m_list.GetSel(count))
			ptrSelected->Push(*ptrIndex);

		count ++;
	}

	CDialog::OnOK();
}

void AtomListDialog::OnAllRadio() 
{
	*ptrShowAtom = IDC_ALL_RADIO;

	LoadAtomList(*ptrShowAtom);
}

void AtomListDialog::OnCaRadio() 
{
	*ptrShowAtom = IDC_CA_RADIO;

	LoadAtomList(*ptrShowAtom);
}

void AtomListDialog::OnHeavyRadio() 
{
	*ptrShowAtom = IDC_HEAVY_RADIO;

	LoadAtomList(*ptrShowAtom);
}
