//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "Qmol.h"
#include "QmolView.h"
#include "AtomRadiusDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAtomRadiusDialog dialog

CAtomRadiusDialog::CAtomRadiusDialog(CView* pParent)
	: CDialog(CAtomRadiusDialog::IDD, pParent)
{
	m_pView = pParent;
	
	atom_num = -1;
	elem = ELEM_C;

	//{{AFX_DATA_INIT(CAtomRadiusDialog)
	m_radius = 0.0f;
	//}}AFX_DATA_INIT
}


void CAtomRadiusDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAtomRadiusDialog)
	DDX_Control(pDX, IDC_ELEMEMT_LIST, m_elem_list);
	DDX_Control(pDX, IDC_ATOM_NUM_EDIT, m_atom_num);
	DDX_Text(pDX, IDC_RADIUS_EDIT, m_radius);
	DDV_MinMaxFloat(pDX, m_radius, 0.f, 100.f);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAtomRadiusDialog, CDialog)
	//{{AFX_MSG_MAP(CAtomRadiusDialog)
	ON_BN_CLICKED(IDC_TEMP_RADIO, OnTempRadio)
	ON_BN_CLICKED(IDC_OCC_RADIO, OnOccRadio)
	ON_BN_CLICKED(IDC_ELEMENT_RADIO, OnElementRadio)
	ON_LBN_SELCHANGE(IDC_ELEMEMT_LIST, OnSelchangeElememtList)
	ON_BN_CLICKED(IDC_DEFAULT_RADIO, OnDefaultRadio)
	ON_BN_CLICKED(IDC_ATOM_RADIO, OnAtomRadio)
	ON_EN_CHANGE(IDC_ATOM_NUM_EDIT, OnChangeAtomNumEdit)
	ON_BN_CLICKED(IDC_APPLY_NOW, OnApplyNow)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAtomRadiusDialog message handlers
BOOL CAtomRadiusDialog::Create() 
{
	return CDialog::Create(CAtomRadiusDialog::IDD);
}

BOOL CAtomRadiusDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// Set up the element list
	m_elem_list.AddString("Carbon");		// ELEM_C
	m_elem_list.AddString("Nitrogen");		// ELEM_N
	m_elem_list.AddString("Oxygen");		// ELEM_O
	m_elem_list.AddString("Sulfur");		// ELEM_S
	m_elem_list.AddString("Hydrogen");		// ELEM_H
	m_elem_list.AddString("Phosphorus");	// ELEM_P
	m_elem_list.AddString("Chlorine");		// ELEM_CL
	m_elem_list.AddString("Zinc");			// ELEM_Z
	m_elem_list.AddString("Sodium");		// ELEM_NA
	m_elem_list.AddString("Iron");			// ELEM_FE
	m_elem_list.AddString("Magnesium");		// ELEM_MG
	m_elem_list.AddString("Potassium");		// ELEM_K
	m_elem_list.AddString("Calcium");		// ELEM_CA
	m_elem_list.AddString("Iodine");		// ELEM_I
	m_elem_list.AddString("Unknown");		// ELEM_DUMMY

	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	int index;
	CString tmpNum;

	switch(pDoc->activeMol->atom_radius_method){
		case QM_RADIUS_ELEMENT:
			m_elem_list.EnableWindow(TRUE);
			m_atom_num.EnableWindow(FALSE);

			m_elem_list.SetCurSel(elem);

			CheckRadioButton(IDC_ELEMENT_RADIO,	// First button
							 IDC_OCC_RADIO,		// Last bottom	
							 IDC_ELEMENT_RADIO);// Button to check	

			m_radius = pDoc->activeMol->elem_radii_scheme[elem];

			break;
		case QM_RADIUS_CUSTOM:
			m_elem_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(TRUE);

			CheckRadioButton(IDC_ELEMENT_RADIO,	// First button
							 IDC_OCC_RADIO,		// Last bottom	
							 IDC_ATOM_RADIO);	// Button to check

			tmpNum.Format("%d", atom_num);

			m_atom_num.SetWindowText(LPCSTR(tmpNum));

			index = pDoc->activeMol->IdToIndex(atom_num);

			if(index != -1){
				m_radius = pDoc->activeMol->elem_radii_scheme[elem];
			}
			else{
				m_radius = 0.0f;
			}
			break;
		case QM_RADIUS_DEFAULT:
			m_elem_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			GetDlgItem(IDC_RADIUS_EDIT)->EnableWindow(FALSE);
			
			CheckRadioButton(IDC_ELEMENT_RADIO,		// First button
							 IDC_OCC_RADIO,			// Last bottom	
							 IDC_DEFAULT_RADIO);	// Button to check
			m_radius = 0.0f;

			break;
		case QM_RADIUS_T:
			m_elem_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			GetDlgItem(IDC_RADIUS_EDIT)->EnableWindow(FALSE);
			
			CheckRadioButton(IDC_ELEMENT_RADIO,	// First button
							 IDC_OCC_RADIO,		// Last bottom	
							 IDC_TEMP_RADIO);	// Button to check
			m_radius = 0.0f;
			break;
		case QM_RADIUS_OCC:
			m_elem_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			GetDlgItem(IDC_RADIUS_EDIT)->EnableWindow(FALSE);
			
			CheckRadioButton(IDC_ELEMENT_RADIO,	// First button
							 IDC_OCC_RADIO,		// Last bottom	
							 IDC_OCC_RADIO);	// Button to check
			m_radius = 0.0f;
			break;
	};

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAtomRadiusDialog::OnTempRadio() 
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->activeMol->atom_radius_method = QM_RADIUS_T;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	GetDlgItem(IDC_RADIUS_EDIT)->EnableWindow(FALSE);
}

void CAtomRadiusDialog::OnOccRadio() 
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->activeMol->atom_radius_method = QM_RADIUS_OCC;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	GetDlgItem(IDC_RADIUS_EDIT)->EnableWindow(FALSE);
}

void CAtomRadiusDialog::OnElementRadio() 
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->activeMol->atom_radius_method = QM_RADIUS_ELEMENT;
	m_elem_list.SetCurSel(elem);

	m_elem_list.EnableWindow(TRUE);
	m_atom_num.EnableWindow(FALSE);
	GetDlgItem(IDC_RADIUS_EDIT)->EnableWindow(TRUE);

	// Update the value of the radius
	m_radius = pDoc->activeMol->elem_radii_scheme[elem];

	UpdateData(FALSE);
}

void CAtomRadiusDialog::OnDefaultRadio() 
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->activeMol->atom_radius_method = QM_RADIUS_DEFAULT;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	GetDlgItem(IDC_RADIUS_EDIT)->EnableWindow(FALSE);
}

void CAtomRadiusDialog::OnAtomRadio() 
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->activeMol->atom_radius_method = QM_RADIUS_CUSTOM;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(TRUE);
	GetDlgItem(IDC_RADIUS_EDIT)->EnableWindow(TRUE);
}


void CAtomRadiusDialog::OnSelchangeElememtList() 
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	elem = m_elem_list.GetCurSel();

	m_radius = pDoc->activeMol->elem_radii_scheme[elem];

	UpdateData(FALSE);
}

void CAtomRadiusDialog::OnChangeAtomNumEdit() 
{
	CString tmpNum;

	m_atom_num.GetWindowText(tmpNum);
	
	atom_num = atoi(LPCSTR(tmpNum));

	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	int index = pDoc->activeMol->IdToIndex(atom_num);

	if(index != -1){
		m_radius = pDoc->activeMol->atoms[index].radius;
	}
	else{
		m_radius = 0.0f;
	}

	UpdateData(FALSE);
}

void CAtomRadiusDialog::ApplyCurrRadius()
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	pDoc->activeMol->SetAtomicRadiiScheme(
		pDoc->activeMol->atom_radius_method, 
		elem, 
		atom_num,
		m_radius);
}

BOOL CAtomRadiusDialog::OnApplyNow() 
{
	// Retrive data from dialog
	if(!UpdateData(TRUE))
		return FALSE;

	ApplyCurrRadius();

	((CQmolView*)m_pView)->InvalidateRect(0, TRUE);

	return TRUE;
}

void CAtomRadiusDialog::OnOK() 
{
	// Make any changes
	if(!OnApplyNow())
		return;

	if(m_pView != NULL){
		m_pView->PostMessage(WM_RADIUS_CLOSE, IDCANCEL);
	}
}

void CAtomRadiusDialog::OnCancel() 
{
	if(m_pView != NULL){
		m_pView->PostMessage(WM_RADIUS_CLOSE, IDCANCEL);
	}
}
