// AtomSelectionDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Qmol.h"
#include "QmolDoc.h"
#include "QmolView.h"
#include "AtomSelectionDialog.h"
#include "AtomListDialog.h"
#include "kabsch.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAtomSelectionDialog dialog


CAtomSelectionDialog::CAtomSelectionDialog(CWnd* pParent)
	: CDialog(CAtomSelectionDialog::IDD, pParent)
{
	pView = (CView*)pParent;

	molA = molB = NULL;

	validAtomSelection = FALSE;

	showAtom = IDC_ALL_RADIO;

	//{{AFX_DATA_INIT(CAtomSelectionDialog)
	m_rmsd = _T("");
	m_mass_weight = TRUE;
	m_auto_rmsd = TRUE;
	//}}AFX_DATA_INIT
}


void CAtomSelectionDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAtomSelectionDialog)
	DDX_Control(pDX, IDC_SEL_LIST, m_sel_list);
	DDX_Control(pDX, IDC_A_COMBO, m_A_mol);
	DDX_Control(pDX, IDC_B_COMBO, m_B_mol);
	DDX_Text(pDX, IDC_RMSD_STATIC, m_rmsd);
	DDX_Check(pDX, IDC_MASS_CHECK, m_mass_weight);
	DDX_Check(pDX, IDC_AUTO_CHECK, m_auto_rmsd);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAtomSelectionDialog, CDialog)
	//{{AFX_MSG_MAP(CAtomSelectionDialog)
	ON_MESSAGE(WM_ATOM_SELECT_UPDATE, UpdateSelection)
	ON_MESSAGE(WM_ATOM_SELECT_RMSD_UPDATE, UpdateRmsd)
	ON_CBN_SELCHANGE(IDC_A_COMBO, OnSelchangeACombo)
	ON_CBN_SELCHANGE(IDC_B_COMBO, OnSelchangeBCombo)
	ON_BN_CLICKED(IDC_ADD_A_BUTTON, OnAddAButton)
	ON_BN_CLICKED(IDC_ADD_B_BUTTON, OnAddBButton)
	ON_BN_CLICKED(IDC_DELETE_A_BUTTON, OnDeleteAButton)
	ON_BN_CLICKED(IDC_DELETE_B_BUTTON, OnDeleteBButton)
	ON_BN_CLICKED(IDC_ALIGN_BUTTON, OnAlignButton)
	ON_BN_CLICKED(IDC_RMSD_BUTTON, OnRmsdButton)
	ON_BN_CLICKED(IDC_AUTO_CHECK, OnAutoCheck)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAtomSelectionDialog message handlers
BOOL CAtomSelectionDialog::Create()
{
	return CDialog::Create(CAtomSelectionDialog::IDD);
}

BOOL CAtomSelectionDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Initialize the sequence list control
	// Build the columns in the parameter list control
	LVCOLUMN tmpCol;
	CDC *pDC;
	RECT listRect;
	int colWidth;

	pDC = m_sel_list.GetDC();
	
	m_sel_list.GetClientRect(&listRect);

	colWidth = (listRect.right - listRect.left)/2;

	tmpCol.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH;
	tmpCol.fmt = LVCFMT_LEFT;
	tmpCol.pszText = "Selected Atoms";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = colWidth;

	tmpCol.iSubItem = 0;	// Ignored
	tmpCol.iImage = 0;		// Ignored
	tmpCol.iOrder = 0;		// Ignored

	m_sel_list.InsertColumn(0, &tmpCol);

	tmpCol.pszText = "Selected Atoms";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = colWidth;

	m_sel_list.InsertColumn(1, &tmpCol);

	m_sel_list.ReleaseDC(pDC);

	// Make selection extend across an entire row
	m_sel_list.SetExtendedStyle(LVS_EX_FULLROWSELECT);
	
	// Pack the list control with sequence data
	UpdateSelection(0, 0);

	LoadComboBox(m_A_mol, molA);
	LoadComboBox(m_B_mol, molB);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

LRESULT CAtomSelectionDialog::UpdateSelection(WPARAM wParam, LPARAM lParam)
{
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);
	
	int i;

	if(pDoc->activeMol == NULL){
		molA = NULL;
		molB = NULL;

		maskA.Alloc(0);
		maskB.Alloc(0);
		m_sel_list.DeleteAllItems();

		while(!selectA.IsEmpty()){
			selectA.Pop();
		}

		while(!selectB.IsEmpty()){
			selectB.Pop();
		}

		LoadComboBox(m_A_mol, molA);
		LoadComboBox(m_B_mol, molB);

		return 0L;
	}

	// Do we need to load new molecules?
	if(!pDoc->IsValidMol(molA)){
		int num;

		molA = pDoc->molList[0];
		maskA.Alloc(num = molA->NumAtoms());
		
		for(i = 0;i < num;i++){
			maskA[i] = FALSE;
		}

		// Remove any existing selection entries
		while(!selectA.IsEmpty()){
			selectA.Pop();
		}
	}

	if(!pDoc->IsValidMol(molB)){
		int num;

		molB = pDoc->molList[0];
		maskB.Alloc(num = molB->NumAtoms());

		for(i = 0;i < num;i++){
			maskB[i] = FALSE;
		}

		// Remove any existing selection entries
		while(!selectB.IsEmpty()){
			selectB.Pop();
		}
	}

	LoadComboBox(m_A_mol, molA);
	LoadComboBox(m_B_mol, molB);

	LoadSelectionList();

	return 0L;
}

void CAtomSelectionDialog::LoadSelectionList()
{
	LVITEM tmpLVITEM;
	CString buffer;

	// Remove all entries from the list control
	m_sel_list.DeleteAllItems();

	// Invalidate any previous selection
	validAtomSelection = FALSE;

	// Zero out the LVITEM structure
	memset(&tmpLVITEM, 0, sizeof(LVITEM));

	int count = 0;

	ListIterator<int> nextA(selectA);
	ListIterator<int> nextB(selectB);
	int *tmpA;
	int *tmpB;

	while((unsigned int)(tmpA = nextA()) + (unsigned int)(tmpB = nextB())){
		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = count;
		tmpLVITEM.iSubItem = 0; // Atoms from A
		tmpLVITEM.state = 0;
		
		if(tmpA){
			buffer.Format("%d:%s:%s%d", 
				molA->atoms[*tmpA].id,
				molA->atoms[*tmpA].name,
				molA->atoms[*tmpA].resName,
				molA->atoms[*tmpA].resSeq);
		}
		else{
			buffer.Format("");
		}

		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		m_sel_list.InsertItem(&tmpLVITEM);

		tmpLVITEM.mask = LVIF_TEXT;
		tmpLVITEM.iItem = count;
		tmpLVITEM.iSubItem = 1; // Atoms from B
		tmpLVITEM.state = 0;
		
		if(tmpB){
			buffer.Format("%d:%s:%s%d", 
				molB->atoms[*tmpB].id,
				molB->atoms[*tmpB].name,
				molB->atoms[*tmpB].resName,
				molB->atoms[*tmpB].resSeq);
		}
		else{
			buffer.Format("");
		}

		tmpLVITEM.pszText = buffer.GetBuffer(0);
		
		m_sel_list.SetItem(&tmpLVITEM);

		count ++;
	}

}

void CAtomSelectionDialog::LoadComboBox(CComboBox &m_combo, 
										molecule *mol)
{
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);

	ListIterator<molecule*> next(pDoc->molList);
	molecule **tmpMol;

	int count = 0;

	m_combo.ResetContent();

	while(tmpMol = next()){
		char *name = strrchr((*tmpMol)->coordName,'\\');
		CString temp;

		if(name)
			temp.Format("%d) %s", count+1, name + 1);
		else
			temp.Format("%d) %s", count+1, (*tmpMol)->coordName);

		m_combo.InsertString(count, temp);

		if((*tmpMol) == mol)
			m_combo.SetCurSel(count);

		count ++;
	}
}

void CAtomSelectionDialog::OnCancel() 
{
	if(pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		pView->PostMessage(WM_ATOM_SELECT_DIALOG, IDCANCEL);
	}
}

void CAtomSelectionDialog::OnSelchangeACombo() 
{
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);

	molecule *tmpMol = pDoc->molList[m_A_mol.GetCurSel()];

	if(tmpMol == molA)
		return;

	molA = tmpMol;

	int num = molA->NumAtoms();

	maskA.Alloc(molA->NumAtoms());

	for(int i = 0;i < num;i++){
		maskA[i] = FALSE;
	}
	
	// Remove any existing selection entries
	while(!selectA.IsEmpty()){
		selectA.Pop();
	}

	LoadSelectionList();
}

void CAtomSelectionDialog::OnSelchangeBCombo() 
{
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);

	molecule *tmpMol = pDoc->molList[m_B_mol.GetCurSel()];

	if(tmpMol == molB)
		return;

	molB = tmpMol;

	int num = molB->NumAtoms();

	maskB.Alloc(molB->NumAtoms());

	for(int i = 0;i < num;i++){
		maskB[i] = FALSE;
	}
	
	// Remove any existing selection entries
	while(!selectB.IsEmpty()){
		selectB.Pop();
	}

	LoadSelectionList();
}

void CAtomSelectionDialog::OnAddAButton() 
{
	list<int> selected;

	AtomListDialog alDlg(FALSE, selected, maskA, molA, showAtom);

	if(alDlg.DoModal() == IDOK){

		ListIterator<int> next(selected);
		int *ptrIndex;
		
		while(ptrIndex = next()){
			selectA.Push(*ptrIndex);

			maskA[*ptrIndex] = TRUE;
		}
			
		LoadSelectionList();
	}
}

void CAtomSelectionDialog::OnAddBButton() 
{
	list<int> selected;

	AtomListDialog alDlg(FALSE, selected, maskB, molB, showAtom);

	if(alDlg.DoModal() == IDOK){
		ListIterator<int> next(selected);
		int *ptrIndex;
		
		while(ptrIndex = next()){
			selectB.Push(*ptrIndex);

			maskB[*ptrIndex] = TRUE;
		}

		LoadSelectionList();
	}
}

void CAtomSelectionDialog::OnDeleteAButton() 
{
	// Delete all selected items
	POSITION pos = m_sel_list.GetFirstSelectedItemPosition();

	if(pos == NULL){
		AfxMessageBox("Please select at least one atom for deletion");
		return;
	}
	
	int len = selectA.Length();

	do{
		int cur_sel = m_sel_list.GetNextSelectedItem(pos);
		if(cur_sel < len){
			int &refPair = selectA[cur_sel];

			maskA[refPair] = FALSE;

			refPair = -1;
		}
	}
	while(pos);

	ListIterator<int> next(selectA);
	int *tmp;

	while(tmp = next()){
		if(*tmp == -1){
			next.RemoveCurrent(selectA);
		}
	}

	LoadSelectionList();
}

void CAtomSelectionDialog::OnDeleteBButton() 
{
	// Delete all selected items
	POSITION pos = m_sel_list.GetFirstSelectedItemPosition();

	if(pos == NULL){
		AfxMessageBox("Please select at least one atom for deletion");
		return;
	}
	
	int len = selectB.Length();

	do{
		int cur_sel = m_sel_list.GetNextSelectedItem(pos);
		if(cur_sel < len){
			int &refPair = selectB[cur_sel];

			maskB[refPair] = FALSE;

			refPair = -1;
		}
	}
	while(pos);

	ListIterator<int> next(selectB);
	int *tmp;

	while(tmp = next()){
		if(*tmp == -1){
			next.RemoveCurrent(selectB);
		}
	}

	LoadSelectionList();
}

void CAtomSelectionDialog::OnAlignButton() 
{
	float rmsd;

	// Invalidate the selection now -- it will be validated if
	// we successfully complete this function.
	validAtomSelection = FALSE;

	if(molA == molB){
		AfxMessageBox("Please select two DIFFERENT molecules!");
		return;
	}

	try{
		if(selectA.Length() != selectB.Length()){
			AfxMessageBox("Please select the same number of "
				"atoms from both molecules");
			return;
		}
		
		if(selectA.Length() < 3){
			AfxMessageBox("Please select at least three atoms from "
				" each molecule");
			return;
		}
		
		AlignMolecules(molA, molB, 
			selectA, selectB,
			m_mass_weight);
		
		rmsd = ComputeRmsd(molA, molB, selectA, selectB,
									   m_mass_weight);
	}
	catch(QmolError &error){
		
		AfxMessageBox(error.message());

		return;
	}
	catch(...){
		AfxMessageBox("Yiikes! An unspecified error has occured while performing "
			"the alignment");
		return;
	}

	validAtomSelection = TRUE;

	// Display the updated rmsd value
	m_rmsd.Format("RMSD = %.3f", rmsd);

	UpdateData(FALSE);

	// Update the view
	((CQmolView*)pView)->postRedisplay();
}

void CAtomSelectionDialog::OnRmsdButton()
{
	float rmsd;

	// Invalidate the selection now -- it will be validated if
	// we successfully complete this function.
	validAtomSelection = FALSE;

	if(molA == molB){
		AfxMessageBox("Please select two DIFFERENT molecules!");
		return;
	}

	// Get the value of the check box
	UpdateData(TRUE);
	
	if(selectA.Length() != selectB.Length()){
		AfxMessageBox("Please select the same number of "
			"atoms from both molecules");
		return;
	}
				
	if(selectA.Length() == 0){
		AfxMessageBox("Please select at least one atom from "
			" each molecule");
		return;
	}
				
	rmsd = ComputeRmsd(molA, molB, selectA, selectB,
		m_mass_weight);
				
	validAtomSelection = TRUE;

	// Display the updated rmsd value
	m_rmsd.Format("RMSD = %.3f", rmsd);

	UpdateData(FALSE);
}

LRESULT CAtomSelectionDialog::UpdateRmsd(WPARAM wParam, LPARAM lParam)
{
	if(!validAtomSelection)
		return 0L;

	OnRmsdButton();

	return 0L;
}

void CAtomSelectionDialog::OnAutoCheck() 
{
	m_auto_rmsd = !m_auto_rmsd;
}

BOOL CAtomSelectionDialog::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	// Don't process IDOK messages sent from the list control
	if(wParam == IDOK)
		return 0;

	return CDialog::OnCommand(wParam, lParam);
}
