//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// Extend the CCommandLineInfo class to provide support
// for addition command line flags and parameters
#ifndef __QMOL_COMMAND_LINE
#define __QMOL_COMMAND_LINE

#include <afxwin.h>
#include "qmol.h"
#include "list.h"

class CQmolCommandLine : public CCommandLineInfo {
public:
	
	CQmolCommandLine()
	{
		m_importPDB = FALSE;
		m_importAUTO = FALSE;
		m_importDCD = FALSE;
		
		m_set_bpp = FALSE;
		m_set_zbuf = FALSE;

		m_traj_kabsch = FALSE;
		m_traj_cm = FALSE;
		m_coord_connect_on_read = TRUE;
		m_multi_traj = FALSE;

		m_help = FALSE;
	};

	// Has the user requested a file import?
	BOOL m_importPDB, m_importDCD, m_importAUTO, 
		m_traj_kabsch, m_traj_cm, m_coord_connect_on_read,
		m_multi_traj, m_help;

	// Has the user requested a specific color depth or Z buffer size?
	BOOL m_set_bpp, m_set_zbuf;

	int m_bpp, m_zbuf;

	// A list of files to import from the command line (wildcards are expanded
	// into a list of strings).
	list<CString> m_import_coord_filename;

	CString m_import_traj_filename;

	void ParseParam( LPCTSTR lpszParam, BOOL bFlag, BOOL bLast );

	int TrajProcess()
	{
		if(m_traj_kabsch)
			return QM_KABSCH;

		if(m_traj_cm)
			return QM_CENTER_ALL;

		return QM_VERBATIM;
	};

	BOOL ConnectOnRead()
	{
		return m_coord_connect_on_read;
	};

	BOOL Help()
	{
		return m_help;
	};
};

#endif