//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
/***********************************************************************/
/* This is the header file for readDCD.c and writeDCD.c. It contains   */
/* Functions for reading and writing the DCD format files used by      */
/* moil and moil-view for storing dynamics coordinates as a function   */
/* of time. Last modified 10/19/96. Written by Jason Gans               */
/***********************************************************************/

#ifndef __DCD
#define	__DCD

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Excepts.h"

// 'Magic' numbers are used to mimic the FORTRAN binary file format
// using C code.

#define	    MAGIC_NUMBER1   84	// Length of header (in bytes)
#define	    MAGIC_NUMBER3   1	// Size of char ?
#define	    MAGIC_NUMBER4   4


// DCD headers are 200 bytes long
#define	    DCD_HEADER_LENGTH	200

#define		DCD_READ	0
#define		DCD_WRITE	1

class dcd{
	private:
		char *error;
		FILE *fin, *fout;
		int swap_endian;
	public:
		dcd()
		{
			fin = fout = NULL;
			error = NULL;
			swap_endian = 0; // FALSE
		};
		
		dcd(const char *filename, int io)
		{
			fin = fout = NULL;
			error = NULL;
			swap_endian = 0; // FALSE
			Open(filename, io);
		};
		
		~dcd()
		{
			Close();
		};
		
		void Close()
		{
			if(fin)
				fclose(fin);
			
			if(fout)
				fclose(fout);
				
			error = NULL;
		};
		
		char* Error()
		{
			return error;
		}
		
		void Open(const char *filename, int io)
		{
			// If the file is open, close it!
			Close();
			
			switch(io){
				case DCD_READ:
					fin = fopen(filename, "r+b");
					if(!fin)
						error = "Can't open DCD file for reading!";
					break;
				case DCD_WRITE:
					fout = fopen(filename, "w+b");
					if(!fout)
						error = "Can't open DCD file for writing!";
					break;
				default:
					throw QMOL_ERROR("Illegal option in dcd Open()");
			};
		};
		
		
		// Functions for READING DCD files:
		void read_dcd_header(int &npt, int &steps, int &size);
		void read_dcd(void *dcd, int num_atom, int size);
	
		// Functions for WRITING DCD files:
		void write_dcd_header(int npt, int steps, int size);
		void write_dcd_step(void *data, int npt, int size);
};
#endif // __DCD
