//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include <math.h>

#include "GLtoPS.h"
#include "Excepts.h"
#include "MainFrm.h"

static char *gouraudtriangleEPS[] =
{
  "/bd{bind def}bind def /triangle { aload pop   setrgbcolor  aload pop 5 3",
  "roll 4 2 roll 3 2 roll exch moveto lineto lineto closepath fill } bd",
  "/computediff1 { 2 copy sub abs threshold ge {pop pop pop true} { exch 2",
  "index sub abs threshold ge { pop pop true} { sub abs threshold ge } ifelse",
  "} ifelse } bd /computediff3 { 3 copy 0 get 3 1 roll 0 get 3 1 roll 0 get",
  "computediff1 {true} { 3 copy 1 get 3 1 roll 1 get 3 1 roll 1 get",
  "computediff1 {true} { 3 copy 2 get 3 1 roll  2 get 3 1 roll 2 get",
  "computediff1 } ifelse } ifelse } bd /middlecolor { aload pop 4 -1 roll",
  "aload pop 4 -1 roll add 2 div 5 1 roll 3 -1 roll add 2 div 3 1 roll add 2",
  "div 3 1 roll exch 3 array astore } bd /gouraudtriangle { computediff3 { 4",
  "-1 roll aload 7 1 roll 6 -1 roll pop 3 -1 roll pop add 2 div 3 1 roll add",
  "2 div exch 3 -1 roll aload 7 1 roll exch pop 4 -1 roll pop add 2 div 3 1",
  "roll add 2 div exch 3 -1 roll aload 7 1 roll pop 3 -1 roll pop add 2 div 3",
  "1 roll add 2 div exch 7 3 roll 10 -3 roll dup 3 index middlecolor 4 1 roll",
  "2 copy middlecolor 4 1 roll 3 copy pop middlecolor 4 1 roll 13 -1 roll",
  "aload pop 17 index 6 index 15 index 19 index 6 index 17 index 6 array",
  "astore 10 index 10 index 14 index gouraudtriangle 17 index 5 index 17",
  "index 19 index 5 index 19 index 6 array astore 10 index 9 index 13 index",
  "gouraudtriangle 13 index 16 index 5 index 15 index 18 index 5 index 6",
  "array astore 12 index 12 index 9 index gouraudtriangle 17 index 16 index",
  "15 index 19 index 18 index 17 index 6 array astore 10 index 12 index 14",
  "index gouraudtriangle 18 {pop} repeat } { aload pop 5 3 roll aload pop 7 3",
  "roll aload pop 9 3 roll 4 index 6 index 4 index add add 3 div 10 1 roll 7",
  "index 5 index 3 index add add 3 div 10 1 roll 6 index 4 index 2 index add",
  "add 3 div 10 1 roll 9 {pop} repeat 3 array astore triangle } ifelse } bd",
  NULL
};

static int compare_back_to_front(const void *a, const void *b)
{
  DepthIndex *p1 = (DepthIndex *) a;
  DepthIndex *p2 = (DepthIndex *) b;
  GLfloat diff = p2->depth - p1->depth;

  if (diff > 0.0) {
    return 1;
  } else if (diff < 0.0) {
    return -1;
  } else {
    return 0;
  }
}

static int compare_front_to_back(const void *a, const void *b)
{
  // The minus sign switches the sorting order
  return -compare_back_to_front(a,b);
}

#define	dot(X1, Y1, X2, Y2)		((X1*X2) + (Y1*Y2))

GLtoPS::GLtoPS(FILE *m_fout, int m_buff_size)
{
	gourand = 0.075f;

	smooth_line = 0.06f;
	lineWidth = 1.0f;
	pointSize = 1.0f;
	occulsion_grid = 200;

	grid_granularity = 500;
	num_grid_x = 10;
	num_grid_y = 10;

	fout = m_fout;

	if(fout == NULL)
		throw QmolError("Invalid file handle", __FILE__, __LINE__);
	
	if(m_buff_size <= 0){
		// Default buffer size = (1024)^2
		feedback_size = 1048576;
	}
	else{
		feedback_size = m_buff_size;
	}

	feedback = new GLfloat [feedback_size];
	
	if(feedback == NULL)
		throw QmolError("Unable to allocate memory for feedback buffer",
			__FILE__, __LINE__);

}

GLtoPS::~GLtoPS()
{
	if(feedback)
		delete [] feedback;
	
}

void GLtoPS::Start()
{
	glFeedbackBuffer(feedback_size, GL_3D_COLOR, feedback);
	glRenderMode(GL_FEEDBACK);
}

BOOL GLtoPS::Stop()
{

	int num_obj = glRenderMode(GL_RENDER);
	
	// Was the buffer big enough to render all of the OpenGL objects?
	if(num_obj < 0){
		
		// Double the buffer size before trying again
		feedback_size *= 2;
		
		if(feedback)
			delete [] feedback;

		feedback = new GLfloat [feedback_size];
		
		if(feedback == NULL)
			throw QmolError("Unable to allocate memory for feedback buffer",
				__FILE__, __LINE__);

		return FALSE;
	}

	WritePS(num_obj);

	return TRUE;
}

void GLtoPS::WritePS(int m_num_obj)
{
	int i;

	// Read back a bunch of OpenGL state to help make the EPS
    // consistent with the OpenGL clear color, line width, point
    // size, and viewport.
	glGetFloatv(GL_VIEWPORT, viewport);
	glGetFloatv(GL_COLOR_CLEAR_VALUE, clearColor);
	glGetFloatv(GL_LINE_WIDTH, &lineWidth);
	glGetFloatv(GL_POINT_SIZE, &pointSize);

	/* Emit EPS header. */
	fputs("%!PS-Adobe-2.0 EPSF-2.0\n", fout);
	/* Notice %% for a single % in the fprintf calls. */
	fprintf(fout, "%%%%Creator: %s (using OpenGL feedback)\n", "Qmol");
	fprintf(fout, "%%%%BoundingBox: %g %g %g %g\n",
		viewport[0], viewport[1], viewport[2], viewport[3]);
	fputs("%%EndComments\n", fout);
	fputs("\n", fout);
	fputs("gsave\n", fout);
	fputs("\n", fout);

	// Output Frederic Delhoume's "gouraudtriangle" PostScript
    // fragment.
	fputs("% the gouraudtriangle PostScript fragement below is free\n", fout);
	fputs("% written by Frederic Delhoume (delhoume@ilog.fr)\n", fout);

	fprintf(fout, "/threshold %g def\n", gourand);
	for (i = 0; gouraudtriangleEPS[i]; i++) {
		fprintf(fout, "%s\n", gouraudtriangleEPS[i]);
	}

	fprintf(fout, "\n%g setlinewidth\n", lineWidth);

	/* Clear the background like OpenGL had it. */
	fprintf(fout, "%g %g %g setrgbcolor\n",
		clearColor[0], clearColor[1], clearColor[2]);

	// Note that Adobe Illustrator 8.0 doesn't like the rectfill command.
	//fprintf(file, "%g %g %g %g rectfill\n\n",
	// viewport[0], viewport[1], viewport[2], viewport[3]);
	// The following block of PostScript is equivalent to the rectfill command:
	fprintf(fout, "gsave\n");
	fprintf(fout, "newpath\n");
	fprintf(fout, "%g %g moveto\n", viewport[0], viewport[1]);
	fprintf(fout, "%g 0 rlineto\n", viewport[2]);
	fprintf(fout, "0 %g rlineto\n", viewport[3]);
	fprintf(fout, "%g neg 0 rlineto\n", viewport[2]);
	fprintf(fout, "closepath\n");
	fprintf(fout, "fill\n");
	fprintf(fout, "grestore\n\n");

	WriteFeedBack(m_num_obj);

	/* Emit EPS trailer. */
	fputs("grestore\n\n", fout);
	fputs("%Add `showpage' to the end of this file to be able to print to a printer.\n",
		fout);
}

void GLtoPS::WriteFeedBack(int m_num_obj)
{
	int token;
	GLfloat *loc, *end;
	Feedback3Dcolor *vertex;
	GLfloat depthSum;
	int nprimitives, item;
	DepthIndex *prims;
	int nvertices, i;

	end = feedback + m_num_obj;

	/* Count how many primitives there are. */
	nprimitives = 0;
	loc = feedback;
	while (loc < end) {
		token = (int)*loc;
		loc++;
		switch (token) {
			case GL_LINE_TOKEN:
			case GL_LINE_RESET_TOKEN:
				loc += 14;
				nprimitives++;
				break;
			case GL_POLYGON_TOKEN:
				nvertices = (int)*loc;
				loc++;
				loc += (7 * nvertices);
				nprimitives++;
				break;
			case GL_POINT_TOKEN:
				loc += 7;
				nprimitives++;
				break;
			case GL_BITMAP_TOKEN:
				loc += 7;
				nprimitives++;
				break;
			default:
				throw QmolError("Unexpected feedback token", 
					__FILE__, __LINE__);
		}
	}

	// Allocate an array of pointers that will point back at
    // primitives in the feedback buffer.  There will be one
    // entry per primitive.  This array is also where we keep the
    // primitive's average depth.  There is one entry per
    // primitive  in the feedback buffer.
	prims = new DepthIndex [nprimitives];

	if(prims == NULL)
		throw QmolError("Unable to allocate memory for PS primitives",
			__FILE__, __LINE__);

	item = 0;
	loc = feedback;
	while (loc < end) {
		token = (int)*loc;
		prims[item].ptr = loc;			/* Save this primitive's location. */
		loc++;
		switch (token) {
			case GL_LINE_TOKEN:
			case GL_LINE_RESET_TOKEN:
				vertex = (Feedback3Dcolor *) loc;
				depthSum = vertex[0].z + vertex[1].z;
				prims[item].depth = depthSum / 2.0f;
				loc += 14;
				break;
			case GL_POLYGON_TOKEN:
				nvertices = (int)*loc;
				loc++;
				vertex = (Feedback3Dcolor *) loc;
				depthSum = vertex[0].z;
				for (i = 1; i < nvertices; i++) {
					depthSum += vertex[i].z;
				}
				prims[item].depth = depthSum / nvertices;
				loc += (7 * nvertices);
				break;
			case GL_POINT_TOKEN:
				vertex = (Feedback3Dcolor *) loc;
				prims[item].depth = vertex[0].z;
				loc += 7;
				break;
			case GL_BITMAP_TOKEN:
				vertex = (Feedback3Dcolor *) loc;
				prims[item].depth = vertex[0].z;
				loc += 7;
				break;
			default:
				throw QmolError("Unknown OpenGL feed back token!", 
					__FILE__, __LINE__);
		}
		item++;
	}

	// First sort the primitives front to back to perpare for occulsion testing
	// via ray tracing
	qsort(prims, nprimitives, sizeof(DepthIndex), compare_front_to_back);

	// Perform occulsion test
	OcclusionTest(prims, nprimitives);

	// Sort the primitives back to front for rendering.
	qsort(prims, nprimitives, sizeof(DepthIndex), compare_back_to_front);

	// Understand that sorting by a primitives average depth
    // doesn't allow us to disambiguate some cases like self
    // intersecting polygons.  Handling these cases would require
    // breaking up the primitives.  That's too involved for this
    // example.  Sorting by depth is good enough for lots of
    // applications.

	// Emit the Encapsulated PostScript for the primitives in
    // back to front order.
	for (item = 0; item < nprimitives; item++) {
		if(prims[item].visible == TRUE){
			WritePrimitive(prims[item].ptr);
		}
	}

  delete [] prims;
}

void GLtoPS::WritePrimitive(GLfloat * loc)
{
	int token;
	int nvertices, i;
	GLfloat red, green, blue;
	int smooth;
	GLfloat dx, dy, dr, dg, db, absR, absG, absB, colormax;
	int steps;
	Feedback3Dcolor *vertex;
	GLfloat xstep, ystep, rstep, gstep, bstep;
	GLfloat xnext, ynext, rnext, gnext, bnext, distance;

	token = (int)*loc;
	loc++;
	switch (token) {
		case GL_LINE_RESET_TOKEN:
		case GL_LINE_TOKEN:
			vertex = (Feedback3Dcolor *) loc;

			dr = vertex[1].red - vertex[0].red;
			dg = vertex[1].green - vertex[0].green;
			db = vertex[1].blue - vertex[0].blue;

			if (dr != 0 || dg != 0 || db != 0) {
				/* Smooth shaded line. */
				dx = vertex[1].x - vertex[0].x;
				dy = vertex[1].y - vertex[0].y;
				distance = (float)sqrt(dx * dx + dy * dy);

				absR = (float)fabs(dr);
				absG = (float)fabs(dg);
				absB = (float)fabs(db);

				colormax = max(absR, max(absG, absB));
				steps = (int)max(1.0f, colormax * distance * smooth_line);

				xstep = dx / steps;
				ystep = dy / steps;

				rstep = dr / steps;
				gstep = dg / steps;
				bstep = db / steps;

				xnext = vertex[0].x;
				ynext = vertex[0].y;
				rnext = vertex[0].red;
				gnext = vertex[0].green;
				bnext = vertex[0].blue;

				// Back up half a step; we want the end points to be
				// exactly the their endpoint colors.
				xnext -= xstep / 2.0f;
				ynext -= ystep / 2.0f;
				rnext -= rstep / 2.0f;
				gnext -= gstep / 2.0f;
				bnext -= bstep / 2.0f;
			} else {
				/* Single color line. */
				steps = 0;
			}

			fprintf(fout, "%g %g %g setrgbcolor\n",
				vertex[0].red, vertex[0].green, vertex[0].blue);
			fprintf(fout, "%g %g moveto\n", vertex[0].x, vertex[0].y);

			for (i = 0; i < steps; i++) {
				xnext += xstep;
				ynext += ystep;
				rnext += rstep;
				gnext += gstep;
				bnext += bstep;
				fprintf(fout, "%g %g lineto stroke\n", xnext, ynext);
				fprintf(fout, "%g %g %g setrgbcolor\n", rnext, gnext, bnext);
				fprintf(fout, "%g %g moveto\n", xnext, ynext);
			}

			fprintf(fout, "%g %g lineto stroke\n", vertex[1].x, vertex[1].y);

			// Each vertex element in the feedback
			// buffer is 7 GLfloats.
			// loc += 14;          

			break;
		case GL_POLYGON_TOKEN:
			nvertices = (int)*loc;
			loc++;
			vertex = (Feedback3Dcolor *) loc;
			
			if (nvertices > 0) {
				red = vertex[0].red;
				green = vertex[0].green;
				blue = vertex[0].blue;
				smooth = 0;
				for (i = 1; i < nvertices; i++) {
					if (red != vertex[i].red || green != vertex[i].green || blue != vertex[i].blue) {
						smooth = 1;
						break;
					}
				}
				if (smooth) {
					/* Smooth shaded polygon; varying colors at vetices. */
					int triOffset;

					/* Break polygon into "nvertices-2" triangle fans. */
					for (i = 0; i < nvertices - 2; i++) {
						triOffset = i * 7;
						fprintf(fout, "[%g %g %g %g %g %g]",
							vertex[0].x, vertex[i + 1].x, vertex[i + 2].x,
							vertex[0].y, vertex[i + 1].y, vertex[i + 2].y);
						fprintf(fout, " [%g %g %g] [%g %g %g] [%g %g %g] gouraudtriangle\n",
							vertex[0].red, vertex[0].green, vertex[0].blue,
							vertex[i + 1].red, vertex[i + 1].green, vertex[i + 1].blue,
							vertex[i + 2].red, vertex[i + 2].green, vertex[i + 2].blue);
					}
				} else {
					/* Flat shaded polygon; all vertex colors the same. */
					fprintf(fout, "newpath\n");
					fprintf(fout, "%g %g %g setrgbcolor\n", red, green, blue);

					/* Draw a filled triangle. */
					fprintf(fout, "%g %g moveto\n", vertex[0].x, vertex[0].y);
					for (i = 1; i < nvertices; i++) {
						fprintf(fout, "%g %g lineto\n", vertex[i].x, vertex[i].y);
					}
					fprintf(fout, "closepath fill\n\n");
				}
			}
			
			// Each vertex element in the
			// feedback buffer is 7 GLfloats.
			//loc += nvertices * 7;	

			break;
		case GL_POINT_TOKEN:
			vertex = (Feedback3Dcolor *) loc;
			fprintf(fout, "%g %g %g setrgbcolor\n", vertex[0].red, vertex[0].green, vertex[0].blue);
			fprintf(fout, "%g %g %g 0 360 arc fill\n\n", vertex[0].x, vertex[0].y, pointSize / 2.0);
			// Each vertex element in the feedback
			// buffer is 7 GLfloats.
			//loc += 7;	
			break;
		case GL_BITMAP_TOKEN:
			// Do nothing for now. Eventually will need to use the pass through functionality
			// to describe the bitmaps for rendering (like storing the CallList id).
			break;
		default:
		  throw QmolError("Unexpected feedback token", __FILE__, __LINE__);
	}
}

void GLtoPS::OcclusionTest(DepthIndex *prims, int num_prim)
{
	// Grid the view-port to efficiently test for polygon
	// occulsion.
	int i, j;

	// The extent of displayed polygons
	int first_time = TRUE;
	GLfloat min_x, max_x;
	GLfloat min_y, max_y;
	GLfloat dx, dy;

	for(i = 0;i < num_prim;i++){
		// Test each polygon in the primitive list. Assume that the 
		// primitive list has already been sorted.
		if((int)*prims[i].ptr != GL_POLYGON_TOKEN){
			// Turn on the display of any non-polygon
			prims[i].visible = TRUE;
			continue;
		}
		
		// Turn OFF the display of any polygon (until turned on later)
		prims[i].visible = FALSE;

		int nvert = (int)*(prims[i].ptr + 1);
		Feedback3Dcolor* vertex = (Feedback3Dcolor *) (prims[i].ptr + 2);
	
		if(first_time){
			first_time = FALSE;
			min_x = max_x = vertex[0].x;
			min_y = max_y = vertex[0].y;

			for(j = 1;j < nvert;j++){
				if(min_x > vertex[j].x)
					min_x = vertex[j].x;
				if(max_x < vertex[j].x)
					max_x = vertex[j].x;
				if(min_y > vertex[j].y)
					min_y = vertex[j].y;
				if(max_y < vertex[j].y)
					max_y = vertex[j].y;
			}
		}
		else{
			for(j = 0;j < nvert;j++){
				if(min_x > vertex[j].x)
					min_x = vertex[j].x;
				if(max_x < vertex[j].x)
					max_x = vertex[j].x;
				if(min_y > vertex[j].y)
					min_y = vertex[j].y;
				if(max_y < vertex[j].y)
					max_y = vertex[j].y;
			}
		}
	}

	// Compute the number of grid elements by requiring approximately
	// "grid_granularity" number of primitives per grid element
	num_grid_y = (int)(sqrt(num_prim/grid_granularity));

	// The minimum grid size is 10 X 10.
	num_grid_y = max(num_grid_y, 10);

	num_grid_x = num_grid_y;

	if(num_grid_x <= 0)
		throw QmolError("Grid size is zero!", 
			__FILE__, __LINE__);


	dx = (max_x - min_x)/num_grid_x;
	dy = (max_y - min_y)/num_grid_y;

	// If we have a 1 dimensional data set, there is no need to grid
	if(dx == 0.0f)
		return;

	if(dy == 0.0f)
		return;

	// Construct a maxtrix of lists of verticies
	list< DepthIndex* > **grid;

	grid = new list< DepthIndex* > *[num_grid_y];

	for(i = 0;i < num_grid_y;i++){
		grid[i] = new list< DepthIndex* > [num_grid_x];
	}

	// Sort the polygons into the different grid elements by vertex
	// position. Polygons that span multiple grid elements are placed into
	// multiple grid elements (i.e. polygon duplication is OK).
	for(i = 0;i < num_prim;i++){
		int id_x, id_y;
		
		DepthIndex* curr_ptr = &(prims[i]);
		DepthIndex* last_ptr = NULL;

		if((int)*curr_ptr->ptr != GL_POLYGON_TOKEN)
			continue;
		
		int nvert = (int)*(curr_ptr->ptr + 1);
		Feedback3Dcolor* vertex = (Feedback3Dcolor *) (curr_ptr->ptr + 2);

	
		GLfloat ave_x = 0.0f;
		GLfloat ave_y = 0.0f; 
		for(j = 0;j < nvert;j++){
			ave_x += vertex[j].x;
			ave_y += vertex[j].y;
		}
	
		ave_x /= nvert;
		ave_y /= nvert;

		id_x = (int)((ave_x - min_x)/dx);
		id_y = (int)((ave_y - min_y)/dy);
			
		// Handle "runt" polygons that lie outside an edge element
		if(id_x == num_grid_x)
			id_x = num_grid_x - 1;

		if(id_y == num_grid_y)
			id_y = num_grid_y - 1;

		int len = grid[id_x][id_y].Length();

		if(len > 0){
			// only check the last pointer in the list
			if(last_ptr != curr_ptr)
				last_ptr = curr_ptr;
				grid[id_x][id_y].PushTail(curr_ptr);
		}
		else{
			last_ptr = curr_ptr;
			grid[id_x][id_y].PushTail(curr_ptr);
		}
	}


	// Keep the user informed
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	if(pFrame){
		pFrame->InitProgress(num_grid_x);
	}

	// Check each grid element
	for(i = 0;i < num_grid_x;i++){
		for(j = 0;j < num_grid_y;j++){
			OcclusionTestGrid(grid[i][j], i*dx + min_x, (i+1)*dx + min_x,
				j*dy + min_y, (j+1)*dy + min_y);
		}

		if(pFrame){
			pFrame->StepProgress();
		}
	}

	if(pFrame){
		pFrame->CloseProgress();
	}

	// Clean up the grid
	for(i = 0;i < num_grid_y;i++){
		delete [] grid[i];
	}	

	delete [] grid;
}

void GLtoPS::OcclusionTestGrid(list< DepthIndex* > &grid_elem, 
							   float min_x, float max_x, float min_y, float max_y)
{
	int len = grid_elem.Length();

	const int ray_density = min(max(len, 50), 100);

	float x, y;
	float dx = (max_x - min_x)/ray_density;
	float dy = (max_y - min_y)/ray_density;

	int i;
	
	for(x = min_x;x < max_x;x += dx){
		for(y = min_y;y < max_y;y += dy){
			for(i = 0;i < len;i++){
				DepthIndex* ptrDI = grid_elem[i];

				int nvert = (int)*(ptrDI->ptr + 1);
				Feedback3Dcolor* vertex = (Feedback3Dcolor *) (ptrDI->ptr + 2);

				if(OcclusionTestPoly(x, y, ptrDI->ptr) == TRUE){
					// If we found an intersecting polygon, stop looping over the
					// sub-loop of polygons
					ptrDI->visible = TRUE;
					break;
				}
			}
		}
	}
}

BOOL GLtoPS::OcclusionTestPoly(GLfloat x, GLfloat y, GLfloat *ptr)
{
	// Skip the primitive token
	ptr++;
	int nvert = (int)*ptr;
	ptr++;
	Feedback3Dcolor* vertex = (Feedback3Dcolor *) ptr;

	if(nvert == 3){
		GLfloat vx0 = vertex[0].x;
		GLfloat dvx1 = vertex[1].x - vx0;
		GLfloat vy0 = vertex[0].y;
		GLfloat dvy1 = vertex[1].y - vy0;

		// This test assumes the same winding for all polygons!
		if((y - vy0)*dvx1 <= (x - vx0)*dvy1)
			return FALSE;

		vx0 = vertex[1].x;
		dvx1 = vertex[2].x - vx0;
		vy0 = vertex[1].y;
		dvy1 = vertex[2].y - vy0;

		// This test assumes the same winding for all polygons!
		if((y - vy0)*dvx1 <= (x - vx0)*dvy1)
			return FALSE;

		vx0 = vertex[2].x;
		dvx1 = vertex[0].x - vx0;
		vy0 = vertex[2].y;
		dvy1 = vertex[0].y - vy0;

		// This test assumes the same winding for all polygons!
		if((y - vy0)*dvx1 <= (x - vx0)*dvy1)
			return FALSE;
	}
	else{
		for(int i = 0;i < nvert;i++){
			GLfloat vx0 = vertex[i].x;
			GLfloat dvx1 = vertex[(i+1)%nvert].x - vx0;
			GLfloat vy0 = vertex[i].y;
			GLfloat dvy1 = vertex[(i+1)%nvert].y - vy0;

			// This test assumes the same winding for all polygons!
			if((y - vy0)*dvx1 <= (x - vx0)*dvy1)
				return FALSE;
		}
	}
	
	// The point {x,y} lies within the polygon
	return TRUE;
}