//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 

#ifndef __GL_TO_PS
#define	__GL_TO_PS

// Include the OpenGL headers
#include <GL\gl.h>
#include <GL\glu.h>
#include "list.h"

/* OpenGL's GL_3D_COLOR feedback vertex format. */
typedef struct _Feedback3Dcolor {
	GLfloat x;
	GLfloat y;
	GLfloat z;
	GLfloat red;
	GLfloat green;
	GLfloat blue;
	GLfloat alpha;
} Feedback3Dcolor;

typedef struct _DepthIndex {
	GLfloat *ptr;
	GLfloat depth;
	GLbyte visible;
} DepthIndex;

class GLtoPS{
private:
	GLfloat *feedback;
	GLint feedback_size;
	FILE *fout;

	GLfloat clearColor[4], viewport[4];
	GLfloat lineWidth, pointSize;
	GLint occulsion_grid;
	GLfloat gourand;
	GLfloat smooth_line;

	GLint grid_granularity;
	GLint num_grid_x;
	GLint num_grid_y;

	void WritePS(int m_num_obj);

	void WriteFeedBack(int m_num_obj);

	void WritePrimitive(GLfloat * loc);

	void OcclusionTest(DepthIndex *prims, int num_prim);
	void OcclusionTestGrid(list<DepthIndex*> &grid_elem,
		float min_x, float max_x, float min_y, float max_y);

	BOOL OcclusionTestPoly(GLfloat x, GLfloat y, GLfloat *ptr);

public:
	GLtoPS(FILE *m_fout, int m_buff_size = -1);
	~GLtoPS();

	void Start();
	BOOL Stop();
};

#endif // __GL_TO_PS