//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#include "stdafx.h"
#include "Qmol.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static UINT indicators[] =
{
	ID_SEPARATOR,	// <--- Display general info here
	ID_SEPARATOR,	// <--- Display atom number here
	ID_SEPARATOR,	// <--- Display the atom type here
	ID_SEPARATOR	// <--- Display the amino acid here
//	ID_INDICATOR_CAPS,
//	ID_INDICATOR_NUM,
//	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	ptrProgress = NULL;
}

CMainFrame::~CMainFrame()
{
	CloseProgress();
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndStatusBar.SetPaneInfo(ATOM_NUM_PANE, m_wndStatusBar.GetItemID(ATOM_NUM_PANE),
		SBPS_NORMAL, 30);

	m_wndStatusBar.SetPaneInfo(ATOM_ID_PANE, m_wndStatusBar.GetItemID(ATOM_ID_PANE),
		SBPS_NORMAL, 30);

	m_wndStatusBar.SetPaneInfo(AMINO_ACID_PANE, m_wndStatusBar.GetItemID(AMINO_ACID_PANE),
		SBPS_NORMAL, 40);

	// Rendering and pipe reading info
	m_wndStatusBar.SetPaneInfo(INFO_PANE, m_wndStatusBar.GetItemID(INFO_PANE),
		SBPS_STRETCH, 0);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// Set the initial window size (and position if needed).
	// It would be nice to express the initial window size 
	// as a fraction of the total screen size.
	cs.cx = 450;
	cs.cy = 450;

	return CFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::InitProgress(int range)
{
	// Create the CProgressCtrl as a child of the status bar positioned
	// over the first pane.
	
	RECT rc;
	m_wndStatusBar.GetItemRect (0, &rc);

	if(ptrProgress)
		delete ptrProgress;

	ptrProgress = new CProgressCtrl;

	VERIFY (ptrProgress->Create(WS_CHILD | WS_VISIBLE | PBS_SMOOTH, rc,
		&m_wndStatusBar, 1));
	ptrProgress->SetRange(0, range);
	ptrProgress->SetStep(1);
}

void CMainFrame::StepProgress()
{
	if(ptrProgress){
		ptrProgress->StepIt();
	}
}

void CMainFrame::SetProgress(int m_pos)
{
	if(ptrProgress){
		ptrProgress->SetPos(m_pos);
	}
}

void CMainFrame::CloseProgress()
{
	if(ptrProgress){
		delete ptrProgress;
		ptrProgress = NULL;
	}
}
