//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#ifndef __QMOL_MEASURE
#define	__QMOL_MEASURE

#include "SelectObject.h"
#include "list.h"

class AtomInfo{
public:
	SelectObj obj;

	int serial;

	int model;	
};

// Store user selections and measurements
class measure {
	private:
		// Up to four atoms can be selected
		list<AtomInfo> m_list;
		
		// The atomic coordinates
		float x[4], y[4], z[4];

		// The four atoms determine up to three bonds
		float bond_length[3];
		
		// The three bonds determine up to two bond angles
		float bond_angle[2];
		
		// The three bonds determine one torison angle
		float torsion_angle;
	public:
	
		measure();
		
		molecule *GetMol(int i);

		float x_coord(int i);
		float y_coord(int i);
		float z_coord(int i);
		int Id(int i);
		int Model(int i);
		int Index(int i);
		unsigned char *Color(int i);

		float Bond(int i);
		float Angle(int i);
		float Torsion();
		
		int NumPicked();
		BOOL Push(SelectObj &m_obj);
		void Update();

		// Remove all measurements
		void Flush();

		// Only remove measurements that include molecules
		// that no longer exist
		void Flush(list<molecule*> &molList);
};

#endif //__QMOL_MEASURE