//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
// 
#include "stdafx.h"
#include "Qmol.h"
#include "QMColorDialog.h"
#include "QmolDoc.h"
#include "QmolView.h"
#include "QmolUtil.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CQMColorDialog dialog


CQMColorDialog::CQMColorDialog(CView* pParent)
	: CDialog(CQMColorDialog::IDD, pParent)
{
	m_pView = pParent;
	
	m_flexClrDlg = new CFlexColorDialog(this, TRUE);

	// Color by element is the default
	color_method = IDC_ELEMENT_RADIO;
	
	atom_num = -1;
	res_num = -1;

	elem = ELEM_C;
	sec_struc = SS_ALPHA_HELIX;

	// The dialog has not been initialized
	is_initialized = FALSE;

	// Default values for the "color clip board"
	valid_clr_buffer = FALSE;
	color_buffer[0] = (char)200;
	color_buffer[1] = (char)200;
	color_buffer[2] = (char)200;
	color_buffer[3] = (char)255;

	//{{AFX_DATA_INIT(CQMColorDialog)
	m_b = _T("");
	m_g = _T("");
	m_r = _T("");
	m_a = _T("");
	//}}AFX_DATA_INIT
}

CQMColorDialog::~CQMColorDialog()
{
	delete m_flexClrDlg;
}

void CQMColorDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CQMColorDialog)
	DDX_Control(pDX, IDC_RES_NUM_EDIT, m_res_num);
	DDX_Control(pDX, IDC_STRUCTURE_LIST, m_struc_list);
	DDX_Control(pDX, IDC_ATOM_NUM_EDIT, m_atom_num);
	DDX_Control(pDX, IDC_ELEMEMT_LIST, m_elem_list);
	DDX_Text(pDX, IDC_B_EDIT, m_b);
	DDV_MaxChars(pDX, m_b, 3);
	DDX_Text(pDX, IDC_G_EDIT, m_g);
	DDV_MaxChars(pDX, m_g, 3);
	DDX_Text(pDX, IDC_R_EDIT, m_r);
	DDV_MaxChars(pDX, m_r, 3);
	DDX_Text(pDX, IDC_A_EDIT, m_a);
	DDV_MaxChars(pDX, m_a, 3);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CQMColorDialog, CDialog)
	//{{AFX_MSG_MAP(CQMColorDialog)
	ON_EN_CHANGE(IDC_ATOM_NUM_EDIT, OnChangeAtomNumEdit)
	ON_BN_CLICKED(IDC_ATOM_RADIO, OnAtomRadio)
	ON_BN_CLICKED(IDC_ELEMENT_RADIO, OnElementRadio)
	ON_BN_CLICKED(ID_APPLY_NOW, OnApplyNow)
	ON_LBN_SELCHANGE(IDC_ELEMEMT_LIST, OnSelchangeElememtList)
	ON_BN_CLICKED(IDC_COLOR_BUTTON, OnColorButton)
	ON_MESSAGE(WM_FLEX_COLOR, OnFlexColorClose)
	ON_BN_CLICKED(IDC_DEFAULT_RADIO, OnDefaultRadio)
	ON_BN_CLICKED(IDC_TEMP_RADIO, OnTempRadio)
	ON_BN_CLICKED(IDC_OCC_RADIO, OnOccRadio)
	ON_BN_CLICKED(IDC_STRUCTURE_RADIO, OnStructureRadio)
	ON_LBN_SELCHANGE(IDC_STRUCTURE_LIST, OnSelchangeStructureList)
	ON_BN_CLICKED(IDC_AMINO_RADIO, OnAminoRadio)
	ON_BN_CLICKED(IDC_RES_NUM_RADIO, OnResNumRadio)
	ON_EN_CHANGE(IDC_RES_NUM_EDIT, OnChangeResNumEdit)
	ON_BN_CLICKED(IDC_SAVE_CLR_BUTTON, OnSaveClrButton)
	ON_BN_CLICKED(IDC_APPLY_CLR_BUTTON, OnApplyClrButton)
	ON_BN_CLICKED(IDC_AMINO_INDEX_RADIO, OnAminoIndexRadio)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CQMColorDialog message handlers
BOOL CQMColorDialog::Create() 
{
	return CDialog::Create(CQMColorDialog::IDD);
}

BOOL CQMColorDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// Set up the element list
	m_elem_list.AddString("Carbon");		// ELEM_C
	m_elem_list.AddString("Nitrogen");		// ELEM_N
	m_elem_list.AddString("Oxygen");		// ELEM_O
	m_elem_list.AddString("Sulfur");		// ELEM_S
	m_elem_list.AddString("Hydrogen");		// ELEM_H
	m_elem_list.AddString("Phosphorus");	// ELEM_P
	m_elem_list.AddString("Chlorine");		// ELEM_CL
	m_elem_list.AddString("Zinc");			// ELEM_Z
	m_elem_list.AddString("Sodium");		// ELEM_NA
	m_elem_list.AddString("Iron");			// ELEM_FE
	m_elem_list.AddString("Magnesium");		// ELEM_MG
	m_elem_list.AddString("Potassium");		// ELEM_K
	m_elem_list.AddString("Calcium");		// ELEM_CA
	m_elem_list.AddString("Iodine");		// ELEM_I
	m_elem_list.AddString("Unknown");		// ELEM_DUMMY

	// Set up the structure list
	m_struc_list.AddString("Alpha Helix");			// SS_ALPHA_HELIX
	m_struc_list.AddString("3-10 Helix");			// SS_310_HELIX
	m_struc_list.AddString("Pi Helix");				// SS_PI_HELIX
	m_struc_list.AddString("Strand (beta sheet)");	// SS_STRAND
	m_struc_list.AddString("Bridge");				// SS_BRIDGE
	m_struc_list.AddString("Coil");					// SS_COIL
	m_struc_list.AddString("Turn");					// SS_TURN
	m_struc_list.AddString("Turn I");				// SS_TURN_1
	m_struc_list.AddString("Turn I'");				// SS_TURN_1P
	m_struc_list.AddString("Turn II");				// SS_TURN_2
	m_struc_list.AddString("Turn II'");				// SS_TURN_2P
	m_struc_list.AddString("Turn IV");				// SS_TURN_4
	m_struc_list.AddString("Turn VIa");				// SS_TURN_6A
	m_struc_list.AddString("Turn VIb");				// SS_TURN_6B
	m_struc_list.AddString("Turn Gamma");			// SS_TURN_GAMMA
	m_struc_list.AddString("Turn Gamma Inv");		// SS_TURN_GAMMA_INV
	m_struc_list.AddString("Unknown");				// SS_DUMMY

	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	int index;
	int chain_index;
	CString tmpNum;

	switch(color_method){
		case IDC_ELEMENT_RADIO:
			m_elem_list.EnableWindow(TRUE);
			m_struc_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			m_res_num.EnableWindow(FALSE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer);

			m_elem_list.SetCurSel(elem);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_ELEMENT_RADIO);		// Button to check	

			m_r.Format("%d", pDoc->color_scheme[elem][0]);
			m_g.Format("%d", pDoc->color_scheme[elem][1]);
			m_b.Format("%d", pDoc->color_scheme[elem][2]);
			m_a.Format("%d", pDoc->color_scheme[elem][3]);			
			break;
		case IDC_STRUCTURE_RADIO:
			m_elem_list.EnableWindow(FALSE);
			m_struc_list.EnableWindow(TRUE);
			m_atom_num.EnableWindow(FALSE);
			m_res_num.EnableWindow(FALSE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer);

			m_struc_list.SetCurSel(sec_struc);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_STRUCTURE_RADIO);		// Button to check	

			m_r.Format("%d", pDoc->color_scheme_struc[sec_struc][0]);
			m_g.Format("%d", pDoc->color_scheme_struc[sec_struc][1]);
			m_b.Format("%d", pDoc->color_scheme_struc[sec_struc][2]);
			m_a.Format("%d", pDoc->color_scheme_struc[sec_struc][3]);
			break;
		case IDC_ATOM_RADIO:
			m_elem_list.EnableWindow(FALSE);
			m_struc_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(TRUE);
			m_res_num.EnableWindow(FALSE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_ATOM_RADIO);			// Button to check

			tmpNum.Format("%d", atom_num);

			m_atom_num.SetWindowText(LPCSTR(tmpNum));

			index = pDoc->activeMol->IdToIndex(atom_num);

			if(index != -1){
				m_r.Format("%d", pDoc->activeMol->atoms[index].rgb_color[0]);
				m_g.Format("%d", pDoc->activeMol->atoms[index].rgb_color[1]);
				m_b.Format("%d", pDoc->activeMol->atoms[index].rgb_color[2]);
				m_a.Format("%d", pDoc->activeMol->atoms[index].rgb_color[3]);
			}
			else{
				m_r.Format("--");
				m_g.Format("--");
				m_b.Format("--");
				m_a.Format("--");
			}
			break;
		case IDC_RES_NUM_RADIO:
			m_elem_list.EnableWindow(FALSE);
			m_struc_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			m_res_num.EnableWindow(TRUE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_RES_NUM_RADIO);		// Button to check

			tmpNum.Format("%d", res_num);

			m_res_num.SetWindowText(LPCSTR(tmpNum));

			// Get the residue index
			index = pDoc->activeMol->ResIdToIndex(res_num, chain_index);

			if(index != -1){
				// Display the color of the alpha carbon
				index = pDoc->activeMol->secondaryStruc[chain_index].
					sequence[index].c_alpha_index;

				m_r.Format("%d", pDoc->activeMol->atoms[index].rgb_color[0]);
				m_g.Format("%d", pDoc->activeMol->atoms[index].rgb_color[1]);
				m_b.Format("%d", pDoc->activeMol->atoms[index].rgb_color[2]);
				m_a.Format("%d", pDoc->activeMol->atoms[index].rgb_color[3]);
			}
			else{
				m_r.Format("--");
				m_g.Format("--");
				m_b.Format("--");
				m_a.Format("--");
			}
			break;
		case IDC_DEFAULT_RADIO:
			m_elem_list.EnableWindow(FALSE);
			m_struc_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			m_res_num.EnableWindow(FALSE);

			GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_DEFAULT_RADIO);		// Button to check

			break;
		case IDC_TEMP_RADIO:
			m_elem_list.EnableWindow(FALSE);
			m_struc_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			m_res_num.EnableWindow(FALSE);

			GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_TEMP_RADIO);			// Button to check
			break;
		case IDC_OCC_RADIO:
			m_elem_list.EnableWindow(FALSE);
			m_struc_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			m_res_num.EnableWindow(FALSE);

			GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_OCC_RADIO);			// Button to check
			break;
		case IDC_AMINO_INDEX_RADIO:
			m_elem_list.EnableWindow(FALSE);
			m_struc_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			m_res_num.EnableWindow(FALSE);

			GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_AMINO_INDEX_RADIO);	// Button to check
			break;
		case IDC_AMINO_RADIO:
			m_elem_list.EnableWindow(FALSE);
			m_struc_list.EnableWindow(FALSE);
			m_atom_num.EnableWindow(FALSE);
			m_res_num.EnableWindow(FALSE);

			GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
			GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);

			CheckRadioButton(IDC_ELEMENT_RADIO,			// First button
							 IDC_AMINO_INDEX_RADIO,		// Last bottom	
							 IDC_AMINO_RADIO);			// Button to check
			break;
	};
	
	UpdateData(FALSE);

	// The dialog has been initialized
	is_initialized = TRUE;

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CQMColorDialog::OnChangeAtomNumEdit() 
{	
	CString tmpNum;

	m_atom_num.GetWindowText(tmpNum);
	
	atom_num = atoi(LPCSTR(tmpNum));

	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	int index = pDoc->activeMol->IdToIndex(atom_num);

	if(index != -1){
		m_r.Format("%d", pDoc->activeMol->atoms[index].rgb_color[0]);
		m_g.Format("%d", pDoc->activeMol->atoms[index].rgb_color[1]);
		m_b.Format("%d", pDoc->activeMol->atoms[index].rgb_color[2]);
		m_a.Format("%d", pDoc->activeMol->atoms[index].rgb_color[3]);

		m_flexClrDlg->SetRGBA(
			pDoc->activeMol->atoms[index].rgb_color[0],
			pDoc->activeMol->atoms[index].rgb_color[1],
			pDoc->activeMol->atoms[index].rgb_color[2],
			pDoc->activeMol->atoms[index].rgb_color[3]);
	}
	else{
		m_r.Format("--");
		m_g.Format("--");
		m_b.Format("--");
		m_a.Format("--");
	}

	UpdateData(FALSE);
}

void CQMColorDialog::OnChangeResNumEdit() 
{
	CString tmpNum;

	m_res_num.GetWindowText(tmpNum);
	
	res_num = atoi(LPCSTR(tmpNum));

	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	int chain_index;

	int index = pDoc->activeMol->ResIdToIndex(res_num, chain_index);

	if(index != -1){
		// Display the color of the alpha carbon
		index = pDoc->activeMol->secondaryStruc[chain_index].
				sequence[index].c_alpha_index;

		m_r.Format("%d", pDoc->activeMol->atoms[index].rgb_color[0]);
		m_g.Format("%d", pDoc->activeMol->atoms[index].rgb_color[1]);
		m_b.Format("%d", pDoc->activeMol->atoms[index].rgb_color[2]);
		m_a.Format("%d", pDoc->activeMol->atoms[index].rgb_color[3]);

		m_flexClrDlg->SetRGBA(
			pDoc->activeMol->atoms[index].rgb_color[0],
			pDoc->activeMol->atoms[index].rgb_color[1],
			pDoc->activeMol->atoms[index].rgb_color[2],
			pDoc->activeMol->atoms[index].rgb_color[3]);
	}
	else{
		m_r.Format("--");
		m_g.Format("--");
		m_b.Format("--");
		m_a.Format("--");
	}

	UpdateData(FALSE);
}

void CQMColorDialog::OnApplyNow() 
{
	// Retreive data from dialog
	if(this->GetSafeHwnd())
		UpdateData(TRUE);

	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);
	
	int i, j, k, index, chain_index;

	switch(color_method){
		case IDC_ELEMENT_RADIO:
			pDoc->color_scheme[elem][0] = atoi(LPCSTR(m_r));
			pDoc->color_scheme[elem][1] = atoi(LPCSTR(m_g));
			pDoc->color_scheme[elem][2] = atoi(LPCSTR(m_b));
			pDoc->color_scheme[elem][3] = atoi(LPCSTR(m_a));

			m_flexClrDlg->SetRGBA(
				pDoc->color_scheme[elem][0],
				pDoc->color_scheme[elem][1],
				pDoc->color_scheme[elem][2],
				pDoc->color_scheme[elem][3]);

			for(i = 0;i < pDoc->activeMol->NumAtoms();i++)
				pDoc->activeMol->atoms[i].SetAtomColor(pDoc->color_scheme);

			break;
		case IDC_STRUCTURE_RADIO:
			pDoc->color_scheme_struc[sec_struc][0] = atoi(LPCSTR(m_r));
			pDoc->color_scheme_struc[sec_struc][1] = atoi(LPCSTR(m_g));
			pDoc->color_scheme_struc[sec_struc][2] = atoi(LPCSTR(m_b));
			pDoc->color_scheme_struc[sec_struc][3] = atoi(LPCSTR(m_a));

			m_flexClrDlg->SetRGBA(
				pDoc->color_scheme_struc[sec_struc][0],
				pDoc->color_scheme_struc[sec_struc][1],
				pDoc->color_scheme_struc[sec_struc][2],
				pDoc->color_scheme_struc[sec_struc][3]);

			pDoc->ColorByStruc();

			break;
		case IDC_ATOM_RADIO:
			index = pDoc->activeMol->IdToIndex(atom_num);

			if(index != -1){
				pDoc->activeMol->atoms[index].rgb_color[0] = atoi(LPCSTR(m_r));
				pDoc->activeMol->atoms[index].rgb_color[1] = atoi(LPCSTR(m_g));
				pDoc->activeMol->atoms[index].rgb_color[2] = atoi(LPCSTR(m_b));
				pDoc->activeMol->atoms[index].rgb_color[3] = atoi(LPCSTR(m_a));

				m_flexClrDlg->SetRGBA(
					pDoc->activeMol->atoms[index].rgb_color[0],
					pDoc->activeMol->atoms[index].rgb_color[1],
					pDoc->activeMol->atoms[index].rgb_color[2],
					pDoc->activeMol->atoms[index].rgb_color[3]);
			}
			break;
		case IDC_RES_NUM_RADIO:
			index = pDoc->activeMol->ResIdToIndex(res_num, chain_index);

			if(index != -1){
				int tmp_r = atoi(LPCSTR(m_r));
				int tmp_g = atoi(LPCSTR(m_g));
				int tmp_b = atoi(LPCSTR(m_b));
				int tmp_a = atoi(LPCSTR(m_a));

				pDoc->activeMol->SetResidueColor(index, chain_index, 
					tmp_r, tmp_g, tmp_b, tmp_a);

				m_flexClrDlg->SetRGBA(
					pDoc->activeMol->atoms[index].rgb_color[0],
					pDoc->activeMol->atoms[index].rgb_color[1],
					pDoc->activeMol->atoms[index].rgb_color[2],
					pDoc->activeMol->atoms[index].rgb_color[3]);
			}
			break;

		case IDC_DEFAULT_RADIO:
			pDoc->SetColorScheme(QM_COLOR_DEFAULT);

			for(i = 0;i < pDoc->activeMol->NumAtoms();i++)
				pDoc->activeMol->atoms[i].SetAtomColor(pDoc->color_scheme);

			break;
		case IDC_TEMP_RADIO:
			for(i = 0;i < pDoc->activeMol->NumAtoms();i++)
				pDoc->activeMol->atoms[i].SetAtomColor(NormalizeByRange(
					pDoc->activeMol->atoms[i].temperature, 
					pDoc->activeMol->min_temp,
					pDoc->activeMol->max_temp));

			break;
		case IDC_OCC_RADIO:
			for(i = 0;i < pDoc->activeMol->NumAtoms();i++)
				pDoc->activeMol->atoms[i].SetAtomColor(NormalizeByRange(
					pDoc->activeMol->atoms[i].occupancy,
					pDoc->activeMol->min_occ,
					pDoc->activeMol->max_occ));

			break;
		case IDC_AMINO_RADIO:
			pDoc->ColorByAmino();
			break;
		case IDC_AMINO_INDEX_RADIO:
			for(i = 0;i < pDoc->activeMol->secondaryStruc.Length();i++){
				array<amino> &seqRef = pDoc->activeMol->secondaryStruc[i].sequence;
				
				int numRes = seqRef.Length();

				for(j = 0;j < numRes;j++){
					for(k = 0;k < seqRef[j].atom_id.Length();k++){
						// Set the color of each atom in an amino acid
						// according to the relative index of the amino
						// acid residue in its chain.
						pDoc->activeMol->atoms[seqRef[j].atom_id[k]].SetAtomColor(
							NormalizeByRange(float(j), 0.0f, float(numRes)));
					}
				}
			}

			break;
	};

	// Recompute the bond color scheme
	// Is a given bond between atoms of the same or different colors?
	pDoc->activeMol->UpdateBondColoring();

	((CQmolView*)m_pView)->InvalidateRect(0, TRUE);
}

void CQMColorDialog::OnSelchangeElememtList() 
{
	if(m_elem_list.GetCurSel() != elem){
		elem = m_elem_list.GetCurSel();
		
		CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
		ASSERT_VALID(pDoc);

		m_r.Format("%d", pDoc->color_scheme[elem][0]);
		m_g.Format("%d", pDoc->color_scheme[elem][1]);
		m_b.Format("%d", pDoc->color_scheme[elem][2]);
		m_a.Format("%d", pDoc->color_scheme[elem][3]);

		m_flexClrDlg->SetRGBA(
			pDoc->color_scheme[elem][0],
			pDoc->color_scheme[elem][1],
			pDoc->color_scheme[elem][2],
			pDoc->color_scheme[elem][3]);

		UpdateData(FALSE);
	}
}

void CQMColorDialog::OnSelchangeStructureList() 
{
	if(m_struc_list.GetCurSel() != sec_struc){
		sec_struc = m_struc_list.GetCurSel();
		
		CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
		ASSERT_VALID(pDoc);

		m_r.Format("%d", pDoc->color_scheme_struc[sec_struc][0]);
		m_g.Format("%d", pDoc->color_scheme_struc[sec_struc][1]);
		m_b.Format("%d", pDoc->color_scheme_struc[sec_struc][2]);
		m_a.Format("%d", pDoc->color_scheme_struc[sec_struc][3]);

		m_flexClrDlg->SetRGBA(
			pDoc->color_scheme_struc[sec_struc][0],
			pDoc->color_scheme_struc[sec_struc][1],
			pDoc->color_scheme_struc[sec_struc][2],
			pDoc->color_scheme_struc[sec_struc][3]);

		UpdateData(FALSE);
	}
}

void CQMColorDialog::OnColorButton() 
{
	// Get the current color from the color text boxes
	UpdateData(TRUE);

	unsigned char r = abs(atoi(LPCSTR(m_r)) % 256);
	unsigned char g = abs(atoi(LPCSTR(m_g)) % 256);
	unsigned char b = abs(atoi(LPCSTR(m_b)) % 256);
	unsigned char a = abs(atoi(LPCSTR(m_a)) % 256);

	m_flexClrDlg->SetRGBA(r, g, b, a);

	if(m_flexClrDlg->GetSafeHwnd() == 0){
		m_flexClrDlg->Create();
	}
}

void CQMColorDialog::OnCancel() 
{
	m_flexClrDlg->DestroyWindow();

	if(m_pView != NULL){
		m_pView->PostMessage(WM_COLOR_CLOSE, IDCANCEL);
	}
}

void CQMColorDialog::OnOK() 
{
	// Make any changes
	OnApplyNow();

	m_flexClrDlg->DestroyWindow();

	if(m_pView != NULL){
		m_pView->PostMessage(WM_COLOR_CLOSE, IDCANCEL);
	}
}

LRESULT CQMColorDialog::OnFlexColorClose(WPARAM wParam, LPARAM lParam) 
{
	int r, g, b, a;

	switch(wParam){
		case IDOK:			
			m_flexClrDlg->GetRGBA(r, g, b, a);
			
			m_r.Format("%d", r);
			m_g.Format("%d", g);
			m_b.Format("%d", b);
			m_a.Format("%d", a);

			UpdateData(FALSE);
			m_flexClrDlg->DestroyWindow();
			break;
		case IDCANCEL:
			m_flexClrDlg->DestroyWindow();
			break;
		case FLEX_COLOR_APPLY:
			m_flexClrDlg->GetRGBA(r, g, b, a);
			
			m_r.Format("%d", r);
			m_g.Format("%d", g);
			m_b.Format("%d", b);
			m_a.Format("%d", a);

			UpdateData(FALSE);
	};

	return 0L;
}

void CQMColorDialog::OnAtomRadio() 
{
	color_method = IDC_ATOM_RADIO;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(TRUE);
	m_res_num.EnableWindow(FALSE);
	m_struc_list.EnableWindow(FALSE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(TRUE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(TRUE);
}

void CQMColorDialog::OnElementRadio() 
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	color_method = IDC_ELEMENT_RADIO;

	m_elem_list.SetCurSel(elem);
	m_elem_list.EnableWindow(TRUE);
	m_atom_num.EnableWindow(FALSE);
	m_res_num.EnableWindow(FALSE);
	m_struc_list.EnableWindow(FALSE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(TRUE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(TRUE);

	m_flexClrDlg->SetRGBA(
		pDoc->color_scheme[elem][0],
		pDoc->color_scheme[elem][1],
		pDoc->color_scheme[elem][2],
		pDoc->color_scheme[elem][3]);

	m_r.Format("%d", pDoc->color_scheme[elem][0]);
	m_g.Format("%d", pDoc->color_scheme[elem][1]);
	m_b.Format("%d", pDoc->color_scheme[elem][2]);
	m_a.Format("%d", pDoc->color_scheme[elem][3]);

	UpdateData(FALSE);
}

void CQMColorDialog::OnStructureRadio() 
{
	CQmolDoc* pDoc = ((CQmolView*)m_pView)->GetDocument();
	ASSERT_VALID(pDoc);

	color_method = IDC_STRUCTURE_RADIO;

	m_struc_list.SetCurSel(sec_struc);
	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	m_res_num.EnableWindow(FALSE);
	m_struc_list.EnableWindow(TRUE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(TRUE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(TRUE);

	m_flexClrDlg->SetRGBA(
		pDoc->color_scheme_struc[sec_struc][0],
		pDoc->color_scheme_struc[sec_struc][1],
		pDoc->color_scheme_struc[sec_struc][2],
		pDoc->color_scheme_struc[sec_struc][3]);

	m_r.Format("%d", pDoc->color_scheme_struc[sec_struc][0]);
	m_g.Format("%d", pDoc->color_scheme_struc[sec_struc][1]);
	m_b.Format("%d", pDoc->color_scheme_struc[sec_struc][2]);
	m_a.Format("%d", pDoc->color_scheme_struc[sec_struc][3]);

	UpdateData(FALSE);
}

void CQMColorDialog::OnDefaultRadio() 
{
	color_method = IDC_DEFAULT_RADIO;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	m_res_num.EnableWindow(FALSE);
	m_struc_list.EnableWindow(FALSE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);
}

void CQMColorDialog::OnAminoIndexRadio() 
{
	color_method = IDC_AMINO_INDEX_RADIO;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	m_res_num.EnableWindow(FALSE);
	m_struc_list.EnableWindow(FALSE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);
}


void CQMColorDialog::OnTempRadio() 
{
	color_method = IDC_TEMP_RADIO;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	m_res_num.EnableWindow(FALSE);
	m_struc_list.EnableWindow(FALSE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);
}

void CQMColorDialog::OnOccRadio() 
{
	color_method = IDC_OCC_RADIO;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	m_res_num.EnableWindow(FALSE);
	m_struc_list.EnableWindow(FALSE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);
}


void CQMColorDialog::OnAminoRadio() 
{
	color_method = IDC_AMINO_RADIO;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	m_res_num.EnableWindow(FALSE);
	m_struc_list.EnableWindow(FALSE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(FALSE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(FALSE);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(FALSE);
}

void CQMColorDialog::OnResNumRadio() 
{
	color_method = IDC_RES_NUM_RADIO;

	m_elem_list.EnableWindow(FALSE);
	m_atom_num.EnableWindow(FALSE);
	m_res_num.EnableWindow(TRUE);
	m_struc_list.EnableWindow(FALSE);

	GetDlgItem(IDC_R_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_G_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_B_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_A_EDIT)->EnableWindow(TRUE);
	GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(TRUE);
	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer);
	GetDlgItem(IDC_SAVE_CLR_BUTTON)->EnableWindow(TRUE);
}

void CQMColorDialog::OnSaveClrButton() 
{
	UpdateData(TRUE);

	unsigned char r = abs(atoi(LPCSTR(m_r)) % 256);
	unsigned char g = abs(atoi(LPCSTR(m_g)) % 256);
	unsigned char b = abs(atoi(LPCSTR(m_b)) % 256);
	unsigned char a = abs(atoi(LPCSTR(m_a)) % 256);

	color_buffer[0] = r;
	color_buffer[1] = g;
	color_buffer[2] = b;
	color_buffer[3] = a;

	GetDlgItem(IDC_APPLY_CLR_BUTTON)->EnableWindow(valid_clr_buffer = TRUE);
}

void CQMColorDialog::OnApplyClrButton() 
{
	if(!valid_clr_buffer)
		return;

	m_r.Format("%d", color_buffer[0]);
	m_g.Format("%d", color_buffer[1]);
	m_b.Format("%d", color_buffer[2]);
	m_a.Format("%d", color_buffer[3]);

	m_flexClrDlg->SetRGBA(
		color_buffer[0],
		color_buffer[1],
		color_buffer[2],
		color_buffer[3]);

	UpdateData(FALSE);

	OnApplyNow();
}
