//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/
//

#include "stdafx.h"
#include "Qmol.h"
#include "QmolView.h"
#include "RenderMovieDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRenderMovieDialog dialog

#define		MAX_FPS		120

CRenderMovieDialog::CRenderMovieDialog(CView* pView)
	: CDialog(CRenderMovieDialog::IDD, NULL)
{
	m_pView = pView;
	render_all = TRUE;
	abortRender = FALSE;

	//{{AFX_DATA_INIT(CRenderMovieDialog)
	m_fps = _T("");
	m_height = _T("");
	m_name = _T("");
	m_start = _T("");
	m_stop = _T("");
	m_width = _T("");
	m_update_surf = FALSE;
	//}}AFX_DATA_INIT
}


void CRenderMovieDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRenderMovieDialog)
	DDX_Control(pDX, IDC_SPIN_WIDTH, m_spin_width);
	DDX_Control(pDX, IDC_SPIN_HEIGHT, m_spin_height);
	DDX_Control(pDX, IDC_SPIN_FPS, m_spin_fps);
	DDX_Control(pDX, IDC_COMBO_COLOR, m_color_depth);
	DDX_Control(pDX, IDC_PROGRESS_MOVIE, m_progress);
	DDX_Text(pDX, IDC_EDIT_FPS, m_fps);
	DDX_Text(pDX, IDC_EDIT_HEIGHT, m_height);
	DDX_Text(pDX, IDC_EDIT_NAME, m_name);
	DDX_Text(pDX, IDC_EDIT_START, m_start);
	DDX_Text(pDX, IDC_EDIT_STOP, m_stop);
	DDX_Text(pDX, IDC_EDIT_WIDTH, m_width);
	DDX_Check(pDX, IDC_SURF_CHECK, m_update_surf);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRenderMovieDialog, CDialog)
	//{{AFX_MSG_MAP(CRenderMovieDialog)
	ON_BN_CLICKED(ID_BUTTON_RECORD, OnButtonRecord)
	ON_BN_CLICKED(IDC_RADIO_ALL, OnRadioAll)
	ON_BN_CLICKED(IDC_RADIO_RANGE, OnRadioRange)
	ON_BN_CLICKED(IDC_BUTTON_BROWSE, OnButtonBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRenderMovieDialog message handlers
BOOL CRenderMovieDialog::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	CQmolView *pView = (CQmolView *)(m_pView);
	
	CQmolDoc* pDoc = pView->GetDocument();
	ASSERT_VALID(pDoc);

	// Set some sensible defaults

	// Start at step one to avoid step zero (which is the structure
	// of the pdb file).
	m_start.Format("1");
	m_stop.Format("%d", pDoc->MaxTrajStep());

	// A nice rate for a movie
	m_fps.Format("15");
	
	m_height.Format("%d", pView->viewHeight);
	m_width.Format("%d", pView->viewWidth);

	// The color depth
	m_color_depth.SetCurSel(COLOR_DEPTH_16);

	// Set the radio buttons
	if(render_all){
		CheckRadioButton(IDC_RADIO_ALL,		// First button
						 IDC_RADIO_RANGE,	// Last bottom	
						 IDC_RADIO_ALL);

		GetDlgItem(IDC_EDIT_START)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_STOP)->EnableWindow(FALSE);
	}
	else{
		CheckRadioButton(IDC_RADIO_ALL,		// First button
						 IDC_RADIO_RANGE,	// Last bottom	
						 IDC_RADIO_RANGE);

		GetDlgItem(IDC_EDIT_START)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_STOP)->EnableWindow(TRUE);
	}

	// Connect the spinners
	m_spin_fps.SetBuddy(GetDlgItem(IDC_EDIT_FPS));
	m_spin_width.SetBuddy(GetDlgItem(IDC_EDIT_WIDTH));
	m_spin_height.SetBuddy(GetDlgItem(IDC_EDIT_HEIGHT));

	// Set the allowed spinner range
	m_spin_fps.SetRange32(1, MAX_FPS);
	m_spin_width.SetRange32(1, UD_MAXVAL);
	m_spin_height.SetRange32(1, UD_MAXVAL);

	UpdateData(FALSE);

	return TRUE;
}

void CRenderMovieDialog::OnButtonRecord() 
{
	UpdateData(TRUE);

	CQmolView *pView = (CQmolView *)(m_pView);
	
	CQmolDoc* pDoc = pView->GetDocument();
	ASSERT_VALID(pDoc);

	int bpp;
	int fps;
	int width, height;

	if(render_all){
		current_step = 1;
		max_step = pDoc->MaxTrajStep();
	}
	else{
		current_step = atoi(m_start);
		max_step = atoi(m_stop);
	}
	
	// Allow the user to specify an movie with a single frame
	// (i.e. max_step == current_step).
	if((current_step < 0) || (max_step < current_step)){
		AfxMessageBox("Please enter a valid step range!");
		return;
	}
	
	m_progress.SetRange32(0, max_step - current_step + 1);
	m_progress.SetStep(1);

	switch(m_color_depth.GetCurSel()){
		case COLOR_DEPTH_16:
			bpp = 16;
			break;
		case COLOR_DEPTH_24:
			bpp = 24;
			break;
	};

	fps = atoi(m_fps);

	if((fps < 1) || (fps >= MAX_FPS)){
		CString errorMsg;

		errorMsg.Format("Please enter a frame rate between 1 and %d", MAX_FPS);
		AfxMessageBox(errorMsg);
		return;
	}

	width = atoi(m_width);

	if(width < 1){
		AfxMessageBox("Please enter a valid width");
		return;
	}

	height = atoi(m_height);

	if(height < 1){
		AfxMessageBox("Please enter a valid height");
		return;
	}

	if(m_name.GetLength() == 0){
		AfxMessageBox("Please enter a file name");
		return;
	}

	// Initialize the movie
	try{
		::SetCursor(LoadCursor(NULL, IDC_WAIT));

		if(movie.Init(m_name, width, height, fps, bpp, pView, m_update_surf) == FALSE){
			// The user has cancled the compression options dialog
			::SetCursor(LoadCursor(NULL, IDC_ARROW));
			return;
		}

		while(current_step <= max_step){
			pDoc->CurrentTrajStep(current_step);
			movie.Render();
			
			current_step++;

			m_progress.StepIt();
		}
		
		movie.Close();
		::SetCursor(LoadCursor(NULL, IDC_ARROW));
	}
	catch(...){
		movie.Close();

		AfxMessageBox("Unable to create movie!");

		::SetCursor(LoadCursor(NULL, IDC_ARROW));
		OnCancel();
		return;
	}

	AfxMessageBox("Movie creation successfull!");
	OnCancel();
}

void CRenderMovieDialog::OnRadioAll() 
{
	if(!render_all){
		render_all = TRUE;
		
		GetDlgItem(IDC_EDIT_START)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_STOP)->EnableWindow(FALSE);
	}
}

void CRenderMovieDialog::OnRadioRange() 
{
	if(render_all){
		render_all = FALSE;
		
		GetDlgItem(IDC_EDIT_START)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_STOP)->EnableWindow(TRUE);
	}
}

void CRenderMovieDialog::OnButtonBrowse() 
{
	CFileDialog fDlg(FALSE /* Save As*/ , "*.avi", m_name);

	fDlg.m_ofn.lpstrTitle = "Specify AVI file to create";
	fDlg.m_ofn.lpstrFilter = "AVI files\0*.avi\0\0";
	fDlg.m_ofn.Flags |= OFN_OVERWRITEPROMPT;

	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0)
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;

	if(fDlg.DoModal() == IDOK){
		m_name = fDlg.GetPathName();

		UpdateData(FALSE);
	}
}

void CRenderMovieDialog::OnCancel() 
{
	CDialog::OnCancel();
}
