//	(c) 2000-2002 J. Gans (jdg9@cornell.edu), Shalloway Lab, 
//	Cornell University. You can modify and freely distribute 
//	this code, but please keep this header intact.
//
//	Look-and-feel inspired by Xmol 
//	(http://www.networkcs.com/msc/docs/xmol/).
//
//	OpenGL selection code and quaterion rotation matrix
//	formulation from molview demo program by Mark Kilgard
//	(http://reality.sgi.com/opengl/OpenGLforX.html). PostScript
//	rendering also from Mark Kilgard.
//
//	The printing routines are based on code from
//	Craig Fahrnbach <craignan@home.com> via
//	Uwe Kotyczka <kotyczka@bnm-gmbh.de>.
//
//	Covalent bond determination code based on VMD
//	http://www.ks.uiuc.edu/Research/vmd/
//
//	Stride (http://www.embl-heidelberg.de/stride/stride.html)
//	is used for secondary structure determination and has been 
//	included in Qmol with the kind permission of 
//	Dmitrij Frishman, PhD
//	Institute for Bioinformatics
//	GSF - Forschungszentrum fr Umwelt und Gesundheit, GmbH
//	Ingolstdter Landstrae 1,
//	D-85764 Neuherberg, Germany
//	Telephone: +49-89-3187-4201
//	Fax: +49-89-31873585
//	e-mail: d.frishman@gsf.de
//	WWW: http://mips.gsf.de/mips/staff/frishman/

#include "stdafx.h"
#include "Qmol.h"
#include "QmolDoc.h"
#include "QmolView.h"
#include "SequenceDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSequenceDialog dialog

static char *ss_types[] = {"AlphaHelix", "310Helix", "PiHelix",
		"Strand", "Bridge", "Coil", "Turn", "TurnI", "TurnI'",
		"TurnII", "TurnII'", "TurnIV", "TurnVIa", "TurnVIb", "TurnVIII",
		"GammaClassic", "GammaInv", "Unknown"};

CSequenceDialog::CSequenceDialog(CWnd* pParent)
	: CDialog(CSequenceDialog::IDD, pParent)
{
	pView = (CView*)pParent;

	// Set up the color dialog
	flexCDlg = new CFlexColorDialog(this);

	// By default, don't use auto apply for now ...
	flexCDlg->DisableAutoApply();

	//{{AFX_DATA_INIT(CSequenceDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CSequenceDialog::~CSequenceDialog()
{
	if(flexCDlg)
		delete flexCDlg;
}

void CSequenceDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSequenceDialog)
	DDX_Control(pDX, IDC_SEQUENCE_LIST, m_list);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSequenceDialog, CDialog)
	//{{AFX_MSG_MAP(CSequenceDialog)
	ON_WM_SIZE()
	ON_WM_SIZING()
	ON_MESSAGE(WM_SEQUENCE_RELOAD, LoadSequenceData)
	ON_BN_CLICKED(ID_BUTTON_COLOR, OnButtonColor)
	ON_MESSAGE(WM_FLEX_COLOR, OnFlexColorMsg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSequenceDialog message handlers
BOOL CSequenceDialog::Create()
{
	return CDialog::Create(CSequenceDialog::IDD);
}

BOOL CSequenceDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Initialize the sequence list  control
	// Build the columns in the parameter list control
	LVCOLUMN tmpCol;
	CDC *pDC;
	float col_width_fudge_factor = 1.2f;
	pDC = m_list.GetDC();

	tmpCol.mask = LVCF_TEXT | LVCF_FMT | LVCF_WIDTH;
	tmpCol.fmt = LVCFMT_LEFT;
	tmpCol.pszText = "Residue";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = (int)(col_width_fudge_factor*
		pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	tmpCol.iSubItem = 0;	// Ignored
	tmpCol.iImage = 0;		// Ignored
	tmpCol.iOrder = 0;		// Ignored

	m_list.InsertColumn(0, &tmpCol);

	tmpCol.pszText = "Chain";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);

	tmpCol.cx = (int)(col_width_fudge_factor*
		pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	m_list.InsertColumn(1, &tmpCol);

	tmpCol.pszText = "Model";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);
	tmpCol.cx = (int)(col_width_fudge_factor*
		pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	m_list.InsertColumn(2, &tmpCol);

	tmpCol.pszText = "Sec Struct";
	tmpCol.cchTextMax = strlen(tmpCol.pszText);
	tmpCol.cx = (int)(col_width_fudge_factor*
		pDC->GetTextExtent(tmpCol.pszText, tmpCol.cchTextMax).cx);

	m_list.InsertColumn(3, &tmpCol);

	m_list.ReleaseDC(pDC);

	// Make selection extend across an entire row
	m_list.SetExtendedStyle(LVS_EX_FULLROWSELECT);

	// Pack the list control with sequence data
	LoadSequenceData(0, 0);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

LRESULT CSequenceDialog::LoadSequenceData(WPARAM wParam, LPARAM lParam)
{
	// Pack the list control with sequence data
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);

	// Remove any pre-existing items from the list control
	m_list.DeleteAllItems();

	if(pDoc->activeMol == NULL){
		// AfxMessageBox("NULL mol pointer in sequence dialog!");
		return 0L;
	}

	// The total number of residues
	int num_residue = 0;

	// Store a list of all the amino acid chains
	list<char> chains;

	CString buffer;
	LVITEM tmpLVITEM;
		
	// Zero out the LVITEM structure
	memset(&tmpLVITEM, 0, sizeof(LVITEM));

	array<amino_chain> &ss = pDoc->activeMol->secondaryStruc;

	int i, j, k;

	for(i = 0;i < ss.Length();i++){
		array<amino> &seq = ss[i].sequence;
		int model_id;
		char chain_id = ss[i].chain;

		// The number of amino acid chains identified so far
		int num_chains = chains.Length();

		// Is this chain ID already in the list of chain id's?
		for(k = 0;k < num_chains;k++){
			if(chains[k] == chain_id)
				break;
		}
		
		if((k < num_chains) || (num_chains == 0)){
			// We found an existing chain! This must be
			// a new model
			model_id = ss[i].model;
		}
		else{
			// This is new chain
			model_id = -1;
			chains.Push(chain_id);
		}

		for(j = 0;j < seq.Length();j++){
			// Add the Residue name
			tmpLVITEM.mask = LVIF_TEXT;
			tmpLVITEM.iItem = num_residue;
			tmpLVITEM.iSubItem = 0; // The item itself
			tmpLVITEM.state = 0;
			tmpLVITEM.stateMask = LVIS_SELECTED;

			buffer.Format("%s %d", seq[j].GetResName(), seq[j].resSeq);
			tmpLVITEM.pszText = buffer.GetBuffer(0);

			m_list.InsertItem(&tmpLVITEM);

			// Add the Chain ID
			tmpLVITEM.mask = LVIF_TEXT;
			tmpLVITEM.iItem = num_residue;
			tmpLVITEM.iSubItem = 1; // The chain id
			tmpLVITEM.state = 0;
			tmpLVITEM.stateMask = LVIS_SELECTED;

			buffer.Format("%c", chain_id);
			tmpLVITEM.pszText = buffer.GetBuffer(0);

			m_list.SetItem(&tmpLVITEM);

			// Add the Model ID
			tmpLVITEM.mask = LVIF_TEXT;
			tmpLVITEM.iItem = num_residue;
			tmpLVITEM.iSubItem = 2; // The model id
			tmpLVITEM.state = 0;
			tmpLVITEM.stateMask = LVIS_SELECTED;

			if(model_id != -1)
				buffer.Format("%d", model_id);
			else
				buffer.Format("");

			tmpLVITEM.pszText = buffer.GetBuffer(0);

			m_list.SetItem(&tmpLVITEM);

			// Add the secondary Structure name
			tmpLVITEM.mask = LVIF_TEXT;
			tmpLVITEM.iItem = num_residue;
			tmpLVITEM.iSubItem = 3; // The secondary Structure
			tmpLVITEM.state = 0;
			tmpLVITEM.stateMask = LVIS_SELECTED;

			tmpLVITEM.pszText = ss_types[seq[j].ss];

			m_list.SetItem(&tmpLVITEM);

			num_residue++;
		}
	}

	return 0L;
}

void CSequenceDialog::OnCancel() 
{
	if(flexCDlg->GetSafeHwnd())
		flexCDlg->DestroyWindow();

	if(pView != NULL){
		// Modeless case -- don't call the base class OnCancel
		pView->PostMessage(WM_SEQUENCE_DIALOG, IDCANCEL);
	}
}

void CSequenceDialog::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	// ***************************
	// Adjust the dialog controls
	// ***************************
	
	// Has the dialog been initialized?
	if(GetDlgItem(IDCANCEL)->GetSafeHwnd() == NULL)
		return;

	// List padding is 10 pixels
	const int padding = 10;

	CRect button_size;
	GetDlgItem(IDCANCEL)->GetClientRect(&button_size);

	if(cx/3 - padding < button_size.Width()){
		GetDlgItem(IDCANCEL)->MoveWindow(
			(cx - padding)/2 - button_size.Width(),
			cy - padding - button_size.Height(), 
			button_size.Width(), button_size.Height());
		
		GetDlgItem(ID_BUTTON_COLOR)->MoveWindow((cx + padding)/2,
			cy - padding - button_size.Height(), 
			button_size.Width(), button_size.Height());
	}
	else{
		GetDlgItem(IDCANCEL)->MoveWindow(cx/3 - button_size.Width()/2,
			cy - padding - button_size.Height(), 
			button_size.Width(), button_size.Height());
		
		GetDlgItem(ID_BUTTON_COLOR)->MoveWindow(2*cx/3  - button_size.Width()/2,
			cy - padding - button_size.Height(), 
			button_size.Width(), button_size.Height());
	}

	m_list.MoveWindow(padding, padding ,
		cx - 2*padding, cy - 3*padding - button_size.Height());

}

void CSequenceDialog::OnSizing(UINT fwSide, LPRECT pRect)
{
	// Minimum X and Y sizes allowed (which is the button size).
	const int min_x = 100;
	const int min_y = 100;

	int dx = pRect->right - pRect->left;
	int dy = pRect->bottom - pRect->top;

	if((dx > min_x) && (dy > min_y)){
		// No size constraint needed
		CDialog::OnSizing(fwSide, pRect);
		return;
	}

	dx = max(dx, min_x);
	dy = max(dy, min_y);

	// Which edge is the user moving?
	switch(fwSide){
		case 1: // Left
			pRect->left = pRect->right - dx;
			break;
		case 2: // Right
			pRect->right  = pRect->left + dx;
			break;
		case 4: // Top left
			pRect->left = pRect->right - dx;
			pRect->top = pRect->bottom - dy;
			break;
		case 5: // Top right
			pRect->right = pRect->left + dx;
			pRect->top = pRect->bottom - dy;
			break;
		case 7: // Bottom left
			pRect->left = pRect->right - dx;
			pRect->bottom = pRect->top + dy;
			break;
		case 8: // Bottom right
			pRect->right = pRect->left + dx;
			pRect->bottom = pRect->top + dy;
			break;
		case 3: // Top
			pRect->top = pRect->bottom - dy;;
			break;
		case 6: // Bottom
			pRect->bottom = pRect->top + dy;
			break;
	};

	CDialog::OnSizing(fwSide, pRect);
}

void CSequenceDialog::OnButtonColor() 
{
	// Get the currently selected item
	POSITION pos = m_list.GetFirstSelectedItemPosition();	

	if(pos == NULL){
		// The user has not selected any residues, so 
		// load default values
		flexCDlg->SetRGBA(255, 255, 255, 255);
		flexCDlg->Create();
		return;
	}

	// Get the first selected item
	int cur_sel = m_list.GetNextSelectedItem(pos);

	// Lookup the color of the first selected item
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);

	if(pDoc->activeMol == NULL){
		AfxMessageBox("NULL mol pointer in sequence dialog!");
		return;
	}

	// The total number of residues
	int num_residue = 0;

	array<amino_chain> &ss = pDoc->activeMol->secondaryStruc;

	for(int i = 0;i < ss.Length();i++){
		array<amino> &seq = ss[i].sequence;
		
		for(int j = 0;j < seq.Length();j++){
			if(cur_sel == num_residue){
				// we found the residue, now we need to get the 
				// color of the alpha carbon.
				GLubyte *rgba = 
					pDoc->activeMol->atoms[seq[j].c_alpha_index].rgb_color;

				if(flexCDlg->GetSafeHwnd() == 0){
					flexCDlg->SetRGBA(rgba[0], rgba[1], rgba[2], rgba[3]);

					flexCDlg->Create();
				}

				return;
			}

			num_residue++;
		}
	}

	AfxMessageBox("Couldn't find selected residue! What trickery is this?");	
}

LRESULT CSequenceDialog::OnFlexColorMsg(WPARAM wParam, LPARAM lParam) 
{
	int tmp_r, tmp_g, tmp_b, tmp_a;

	CString buffer;

	switch(wParam){
		case IDOK:			
			flexCDlg->GetRGBA(tmp_r, tmp_g, tmp_b, tmp_a);

			flexCDlg->DestroyWindow();
			break;
		case IDCANCEL:
			flexCDlg->DestroyWindow();

			// We don't need to apply a color, so return right away.
			return 0L;
			break;
		case FLEX_COLOR_APPLY:
			flexCDlg->GetRGBA(tmp_r, tmp_g, tmp_b, tmp_a);
			break;
		default:
			// We don't need to apply a color, so return right away.
			return 0L;
	};

	// Get the currently selected item
	POSITION pos = m_list.GetFirstSelectedItemPosition();	

	if(pos == NULL){
		// This error message is skipped to prevent from 
		// cluttering up the screen with error messages 
		// when the user has auto_apply turned on for the 
		// color dialog.
		//AfxMessageBox("Please select at least one residue");
		return 0L;
	}

	// Get the first selected item
	int cur_sel = m_list.GetNextSelectedItem(pos);

	// Lookup the color of the first selected item
	CQmolDoc* pDoc = (CQmolDoc*)(pView->GetDocument());
	ASSERT_VALID(pDoc);

	if(pDoc->activeMol == NULL){
		AfxMessageBox("NULL mol pointer in sequence dialog!");
		return 0L;
	}

	// The total number of residues
	int num_residue = 0;

	array<amino_chain> &ss = pDoc->activeMol->secondaryStruc;

	for(int i = 0;i < ss.Length();i++){
		array<amino> &seq = ss[i].sequence;
		
		for(int j = 0;j < seq.Length();j++){
			if(cur_sel == num_residue){
				// we found the residue
				list<int> &atom_id_ref = seq[j].atom_id;

				for(int k = 0;k < atom_id_ref.Length();k++){
					// Grab a pointer to the color array
					GLubyte *rgba = 
						pDoc->activeMol->atoms[atom_id_ref[k]].rgb_color;

					rgba[0] = tmp_r;
					rgba[1] = tmp_g;
					rgba[2] = tmp_b;
					rgba[3] = tmp_a;
				}

				// Get the selected item
				if(pos == NULL){
					// If pos is NULL, we have iterated through the
					// entire list of selected items

					// Recompute the bond color scheme
					// Is a given bond between atoms of the same or different colors?
					pDoc->activeMol->UpdateBondColoring();

					// Redraw the main windows
					((CQmolView*)pView)->InvalidateRect(0, TRUE);
					return 0L;
				}
				else{
					cur_sel = m_list.GetNextSelectedItem(pos);
				}
			}
			num_residue++;
		}
	}

	AfxMessageBox("Failed to find all selected residues!");
	return 0L;
}

BOOL CSequenceDialog::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	// Don't process IDOK messages sent from the list control
	if(wParam == IDOK)
		return 0;

	return CDialog::OnCommand(wParam, lParam);
}
