// SurfColorProperty.cpp : implementation file
//

#include "stdafx.h"
#include "Qmol.h"
#include "SurfColorProperty.h"
#include "SurfDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSurfColorProperty property page

IMPLEMENT_DYNCREATE(CSurfColorProperty, CPropertyPage)

CSurfColorProperty::CSurfColorProperty() : CPropertyPage(CSurfColorProperty::IDD)
{
	//{{AFX_DATA_INIT(CSurfColorProperty)
	m_enable_alpha = FALSE;
	m_alpha = 255;
	m_auto_apply = TRUE;
	m_alpha_text = _T("");
	m_a = 0;
	m_b = 0;
	m_g = 0;
	m_r = 0;
	//}}AFX_DATA_INIT

	m_pWnd = NULL;

	color_style = COLOR_BY_ATOM;

	m_flexClrDlg = new CFlexColorDialog(this, TRUE);
}

CSurfColorProperty::~CSurfColorProperty()
{
	if(m_flexClrDlg)
		delete m_flexClrDlg;
}

void CSurfColorProperty::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSurfColorProperty)
	DDX_Check(pDX, IDC_ALPHA_CHECK, m_enable_alpha);
	DDX_Slider(pDX, IDC_ALPHA_SLIDER, m_alpha);
	DDX_Check(pDX, IDC_AUTO_APPLY, m_auto_apply);
	DDX_Text(pDX, IDC_ALPHA_TEXT, m_alpha_text);
	DDX_Text(pDX, IDC_A_EDIT, m_a);
	DDV_MinMaxInt(pDX, m_a, -2147483647, 2147483647);
	DDX_Text(pDX, IDC_B_EDIT, m_b);
	DDV_MinMaxInt(pDX, m_b, -2147483647, 2147483647);
	DDX_Text(pDX, IDC_G_EDIT, m_g);
	DDV_MinMaxInt(pDX, m_g, -2147483647, 2147483647);
	DDX_Text(pDX, IDC_R_EDIT, m_r);
	DDV_MinMaxInt(pDX, m_r, -2147483647, 2147483647);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSurfColorProperty, CPropertyPage)
	//{{AFX_MSG_MAP(CSurfColorProperty)
	ON_BN_CLICKED(IDC_ALPHA_CHECK, OnAlphaCheck)
	ON_BN_CLICKED(IDC_AUTO_APPLY, OnAutoApply)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_ATOM_RADIO, OnAtomRadio)
	ON_BN_CLICKED(IDC_SURF_RADIO, OnSurfRadio)
	ON_MESSAGE(WM_FLEX_COLOR, OnFlexColor)
	ON_BN_CLICKED(IDC_COLOR_BUTTON, OnColorButton)
	ON_EN_CHANGE(IDC_R_EDIT, OnChangeREdit)
	ON_EN_CHANGE(IDC_G_EDIT, OnChangeGEdit)
	ON_EN_CHANGE(IDC_B_EDIT, OnChangeBEdit)
	ON_EN_CHANGE(IDC_A_EDIT, OnChangeAEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSurfColorProperty message handlers

BOOL CSurfColorProperty::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	// Set the range of the alpha slider
	((CSliderCtrl*)GetDlgItem(IDC_ALPHA_SLIDER))->SetRange(0, 255);
	((CSliderCtrl*)GetDlgItem(IDC_ALPHA_SLIDER))->SetPos(m_alpha);

	UpdateUI();

	return TRUE;
}

void CSurfColorProperty::UpdateUI()
{
	// Set the radio buttons 
	switch(color_style){
		case COLOR_BY_ATOM:
			CheckRadioButton(IDC_ATOM_RADIO,		// First button
							 IDC_SURF_RADIO,		// Last bottom	
							 IDC_ATOM_RADIO);		// Button to check

			// Color-by-atom options (enable and conditionally enable/disable)
			GetDlgItem(IDC_ALPHA_CHECK)->EnableWindow(TRUE);

			if(m_enable_alpha){
				GetDlgItem(IDC_ALPHA_SLIDER)->EnableWindow(TRUE);
			}
			else{
				GetDlgItem(IDC_ALPHA_SLIDER)->EnableWindow(FALSE);
			}

			// User-color options (disable)
			GetDlgItem(IDC_R_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_G_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_B_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_A_EDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(FALSE);

			break;
		case COLOR_USER:
			CheckRadioButton(IDC_ATOM_RADIO,		// First button
							 IDC_SURF_RADIO,		// Last bottom	
							 IDC_SURF_RADIO);		// Button to check

			// Color-by-atom options (disable)
			GetDlgItem(IDC_ALPHA_CHECK)->EnableWindow(FALSE);

			GetDlgItem(IDC_ALPHA_SLIDER)->EnableWindow(FALSE);

			// User-color options (enable)
			GetDlgItem(IDC_R_EDIT)->EnableWindow(TRUE);
			GetDlgItem(IDC_G_EDIT)->EnableWindow(TRUE);
			GetDlgItem(IDC_B_EDIT)->EnableWindow(TRUE);
			GetDlgItem(IDC_A_EDIT)->EnableWindow(TRUE);
			GetDlgItem(IDC_COLOR_BUTTON)->EnableWindow(TRUE);
			break;
	};

	m_alpha_text.Format("Alpha = %d", m_alpha);

	UpdateData(FALSE);	
}

void CSurfColorProperty::OnAlphaCheck() 
{
	UpdateData();

	UpdateUI();

	if(m_auto_apply)
		ApplyColor();
}

void CSurfColorProperty::OnAutoApply() 
{
	UpdateData();
}

void CSurfColorProperty::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{	
	UpdateData();

	UpdateUI();
	
	if(m_auto_apply)
		ApplyColor();

	CPropertyPage::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CSurfColorProperty::OnAtomRadio() 
{
	if(color_style != COLOR_BY_ATOM){
		color_style = COLOR_BY_ATOM;
		UpdateUI();

		if(m_auto_apply)
			ApplyColor();
	}
}

void CSurfColorProperty::OnSurfRadio() 
{
	if(color_style != COLOR_USER){
		color_style = COLOR_USER;
		UpdateUI();

		if(m_auto_apply)
			ApplyColor();
	}
}

LRESULT CSurfColorProperty::OnFlexColor(WPARAM wParam, LPARAM lParam) 
{
	int r, g, b, a;

	switch(wParam){
		case IDOK:			
			m_flexClrDlg->GetRGBA(r, g, b, a);
			
			m_r = r;
			m_g = g;
			m_b = b;
			m_a = a;

			UpdateData(FALSE);
			
			if(m_auto_apply)
				ApplyColor();

			m_flexClrDlg->DestroyWindow();
			break;
		case IDCANCEL:
			m_flexClrDlg->DestroyWindow();
			break;
		case FLEX_COLOR_APPLY:
			m_flexClrDlg->GetRGBA(r, g, b, a);
			
			m_r = r;
			m_g = g;
			m_b = b;
			m_a = a;

			UpdateData(FALSE);

			if(m_auto_apply)
				ApplyColor();
	};

	return 0L;
}

void CSurfColorProperty::OnColorButton() 
{
	// Get the current color from the color text boxes
	UpdateData(TRUE);

	m_flexClrDlg->SetRGBA(m_r, m_g, m_b, m_a);

	if(m_flexClrDlg->GetSafeHwnd() == 0){
		m_flexClrDlg->Create();
	}
}

void CSurfColorProperty::ApplyColor()
{
	// Send a message to the parent of the property sheet to apply color 
	// changes
	if(m_pWnd)
		m_pWnd->PostMessage(WM_SURF_OPT, ID_SURF_APPLY);
}

void CSurfColorProperty::OnChangeREdit() 
{
	CString buffer;

	// Get the current color from the color text boxes
	GetDlgItem(IDC_R_EDIT)->GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp >= 0) && (tmp <= 255)){
		// Apply the new number
		m_r = tmp;
		UpdateData();
	}
	else{
		// Restore the old number
		UpdateData(FALSE);
	}

	

	if(m_auto_apply)
		ApplyColor();
}

void CSurfColorProperty::OnChangeGEdit() 
{
	CString buffer;

	// Get the current color from the color text boxes
	GetDlgItem(IDC_G_EDIT)->GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp >= 0) && (tmp <= 255)){
		// Apply the new number
		m_g = tmp;
		UpdateData();
	}
	else{
		// Restore the old number
		UpdateData(FALSE);
	}

	if(m_auto_apply)
		ApplyColor();
	
}

void CSurfColorProperty::OnChangeBEdit() 
{
	CString buffer;

	// Get the current color from the color text boxes
	GetDlgItem(IDC_B_EDIT)->GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp >= 0) && (tmp <= 255)){
		// Apply the new number
		m_b = tmp;
		UpdateData();
	}
	else{
		// Restore the old number
		UpdateData(FALSE);
	}

	if(m_auto_apply)
		ApplyColor();
}

void CSurfColorProperty::OnChangeAEdit() 
{
	CString buffer;

	// Get the current color from the color text boxes
	GetDlgItem(IDC_A_EDIT)->GetWindowText(buffer);

	int tmp = atoi(LPCSTR(buffer));

	if((tmp >= 0) && (tmp <= 255)){
		// Apply the new number
		m_a = tmp;
		UpdateData();
	}
	else{
		// Restore the old number
		UpdateData(FALSE);
	}

	if(m_auto_apply)
		ApplyColor();
}
